/*
 * 
  Package: MAG - VistA Imaging
  WARNING: Per VHA Directive 2004-038, this routine should not be modified.
  Date Created: March 11, 2008
  Site Name:  Washington OI Field Office, Silver Spring, MD
  Developer:        BUCKD
  Description: 

        ;; +--------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +--------------------------------------------------------------------+

 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using log4net;
using System.Diagnostics;
using System.Management;

namespace Dexter
{
    /// <summary>
    /// This delegate provides the means for business layer code to write a message to the textBoxInfo control. It maps to the
    /// Info method.
    /// </summary>
    /// <param name="infoMessage">The message string to be displayed on a new textbox line.</param>
    /// <returns></returns>
    public delegate String InfoDelegate(String infoMessage);
    /// <summary>
    /// This delegate provides the means for business layer code to display a filename in the status bar control. It maps to the
    /// ProcessFile method.
    /// </summary>
    /// <param name="filespec">The filename that is currently being examined on the DEX drive.</param>
    public delegate void ProcessFileDelegate(String filespec); // delegate to display filespec currently being processed
    
    /// <summary>
    /// The main form of the Dexter application. Dexter must be installed and run the on a server hosting the EMC DiskXtender
    /// software. When it is run, Dexter automatically detects any jukebox shares on the local server and then generates
    /// the following reports:
    /// - A Platter Summary report that lists the name, serial number and status (online or offline) of every platter managed
    ///   by the jukebox that contains VistA Imaging files.  The information in this report is obtained from the Windows Server
    ///   2003 registry.
    /// - A Platter Detail report for each platter on the jukebox.  These reports are generated by traversing through all the
    ///   files on every network share mapped to the Disk Extender (DEX) drive. Dexter examines each file to determine which
    ///   platter the file resides on, and then writes the filename as a UNC path to the appropriate Platter Detail report file.
    /// </summary>
    public partial class MainForm : Form
    {
        /// <summary>
        /// The MainForm constructor.
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        private const String CANCEL = "Ca&ncel"; // The buttonXrefDirectory text while reports are running
        private JukeboxHelper jukebox = null; // Helper object to handle jukebox related tasks
        private SortedList<String, NetworkShare> dexShares = null;
        private string exitMessageTitle = null;
        private string exitMessage = null;

        #region private methods
        /// <summary>
        /// Returns a log4net logger.
        /// </summary>
        /// <returns></returns>
        private ILog Logger()
        {
            return LogManager.GetLogger(this.GetType().Name);
        }

        /// <summary>
        /// Write a message line to the textBoxInfo control. This method is designed to be used from a worker thread.
        /// </summary>
        /// <param name="infoMessage">The message line to be written.</param>
        /// <returns></returns>
        private String Info(String infoMessage)
        {
            if (this.textBoxInfo.InvokeRequired)
            {
                InfoDelegate d = new InfoDelegate(this.Info);
                this.Invoke(d, new object[] { infoMessage });
            }
            else
            {
                this.textBoxInfo.AppendText(infoMessage + Environment.NewLine);
            }
            Application.DoEvents();
            return infoMessage;
        }

        /// <summary>
        /// Update the labelCurrentFile control on the status bar with the current file being processed. This method is designed
        /// to be used from a worker thread.
        /// </summary>
        private void ProcessFile(String filespec)
        {
            if (this.statusStrip.InvokeRequired)
            {
                ProcessFileDelegate d = new ProcessFileDelegate(this.ProcessFile);
                this.Invoke(d, new object[] { filespec });
            }
            else
            {
                this.toolStripStatusLabelCurrentFile.Text = filespec;
            }
            Application.DoEvents();
        }

        /// <summary>
        /// Ask the user if they want to cancel the current task.
        /// </summary>
        /// <returns>Returns true to cancel the current task, false otherwise.</returns>
        private bool ConfirmCancel()
        {
            DialogResult result = MessageBox.Show("Do you really want to cancel?", "Confim Cancel",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            return result == DialogResult.Yes ? true : false;
        }

        /// <summary>
        /// If the report output directory already contains DEX platter reports, ask if it is OK to overwrite
        /// existing detail report files, and if so delete them.
        /// </summary>
        /// <param name="folder">the output directory</param>
        /// <returns>true if it is OK to delete existing platter reports, false otherwise</returns>
        private bool ConfirmReplace(String folder)
        {
            bool doit = true;
            String[] files = Directory.GetFiles(folder, JukeboxHelper.PLATTER_DETAIL_REPORT_PREFIX + "*.txt", SearchOption.TopDirectoryOnly);

            if (files.Length > 0 || File.Exists(Path.Combine(folder, JukeboxHelper.PLATTER_SUMMARY_REPORT_FILENAME)))
            {
                StringBuilder sb = new StringBuilder();
                sb.AppendFormat("Platter reports already exist in {0}. Dexter will delete these existing reports.", folder);
                sb.AppendLine();
                sb.AppendLine("Do you wish to continue?");

                DialogResult result = MessageBox.Show(sb.ToString(),
                    "Delete existing reports?", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (result == DialogResult.No)
                {
                    doit = false;
                }
                else
                {
                    foreach (String file in files) // delete the existing detailed platter report files
                    {
                        File.Delete(file); 
                    }
                    this.Logger().Info(this.Info("Deleted existing detailed platter reports in " + folder));
                    string networkSharesProcessedFilespec = Path.Combine(folder, JukeboxHelper.NETWORK_SHARES_PROCESSED_TRACKING_FILENAME);
                    if (File.Exists(networkSharesProcessedFilespec))
                    {
                        File.Delete(networkSharesProcessedFilespec);
                    }
                }
            }

            return doit;
        }

        /// <summary>
        /// Display an About dialog box.
        /// </summary>
        private void AboutDexter()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("VistA Imaging MagDexter version {0}.", Application.ProductVersion);
            sb.AppendLine();
            sb.AppendLine();
            sb.AppendLine("DiskXtender is a registered trademark of Legato Systems, Inc.");

            MessageBox.Show(sb.ToString(),
                "MagDexter - The DiskXtender Platter Report Utility", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// Return the selected NetworkShare based on the share path.
        /// </summary>
        /// <param name="sharePath">The fully qualified share path.</param>
        /// <returns>The NetworkShare associated with passed path.</returns>
        //private NetworkShare GetNetworkShareFromPath(String sharePath)
        //{
        //    NetworkShare selectedShare = null;
        //    foreach (NetworkShare dexShare in this.dexShares)
        //    {
        //        if (dexShare.SharePath == sharePath)
        //        {
        //            selectedShare = dexShare;
        //        }
        //    }
        //    return selectedShare;
        //}

        private bool AreMultipleSharesMappedToSamePath()
        {
            bool isMultiple = false;

            foreach (KeyValuePair<String, NetworkShare> keyValue in dexShares)
            {
                if (keyValue.Value.IsDuplicate)
                {
                    isMultiple = true;
                    break;
                }
            }

            return isMultiple;
        }

        /// <summary>
        /// Populates the passed DexShareDataTable with a list of share names and physical paths associated with the
        /// DEX drive. The main form binds this information to a combo box so that the user can specify where to
        /// start scanning on the DEX drive to generate platter reports.
        /// </summary>
        /// <param name="dt"></param>
        private void PopulateDexShareTable()
        {
            DataSetDex.DexShareDataTable dt = this.dataSetDex1.DexShare;
            dt.Clear();
            if (this.AreMultipleSharesMappedToSamePath() == false)
            {
                dt.AddDexShareRow(JukeboxHelper.ALL_NETWORK_SHARES, " All network shares");
            }

            foreach (KeyValuePair<String, NetworkShare> keyValue in dexShares)
            {
                dt.AddDexShareRow(keyValue.Value.ShareName, keyValue.Value.ShareName + " (" + keyValue.Value.SharePath + ")");
            }
        }

        #endregion

        #region events

        /// <summary>
        /// Perform initialization required at form load time. Currently this means instantiate a JukeboxHelper and
        /// use it to populate the dataSetDex.DexShare table with network shares mapped to the DEX drives. If no DEX drive network
        /// shares are located or the version of DiskXtender is not supported then the buttonPlatterReports control will be disabled.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            //this.Text += " Version " + Application.ProductVersion;
            NtDllHelper.info = new InfoDelegate(this.Info);
            this.jukebox = new JukeboxHelper(this.Info, this.ProcessFile);
            this.dexShares = this.jukebox.GetDexShares();
            this.PopulateDexShareTable();
            this.comboBoxNetworkShare.SelectedIndex = 0; // select first item in sorted order

            if (this.dexShares.Count == 0)
            {
                MessageBox.Show("There were no DEX drives detected. Dexter must be run on the same system as DiskXtender. Application will close.",
                    "DiskXtender not found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }
            else if (this.jukebox.IsDexVersionSupported() == false)
            {
                MessageBox.Show("Dexter only supports version 5 of DiskXTender. Application will close.",
                    "Incompatible DiskXtender version found.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }
            else // report on shares that will be processed
            {
                //this.comboBoxDexShares.SelectedIndex = 0; // select first item in sorted order
                this.Info("Welcome to MagDexter.");
                this.Info("");
                this.Info("The following network shares can be scanned to create Platter Reports:");
                foreach (KeyValuePair<String, NetworkShare> keyValue in dexShares)
                {
                    if (keyValue.Value.IsDuplicate)
                    {
                        this.Info("   " + keyValue.Value.ShareName + " (" + keyValue.Value.SharePath + ") - Duplicate");
                    }
                    else
                    {
                        this.Info("   " + keyValue.Value.ShareName + " (" + keyValue.Value.SharePath + ")");
                    }
                }
                this.Info("");
                if (this.AreMultipleSharesMappedToSamePath())
                {
                    this.Info("WARNING");
                    this.Info("Multiple Network Shares are mapped to the same DEX directory.");
                    this.Info("For MagKat to function properly, be sure to only generate reports for network shares that are available in the VistA NETWORK LOCATION file.");
                    this.Info("The option to generate reports for all network shares has been disabled. Reports will need to be generated one network share at a time.");
                    this.Info("");
                }
            }
        }

        /// <summary>
        /// Displays a folder browser dialog to select the report output directory.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSelectOutputFolder_Click(object sender, EventArgs e)
        {
            DialogResult result = this.folderBrowserDialogXrefDir.ShowDialog();
            if (result == DialogResult.OK)
            {
                this.textBoxOutputFolder.Text = this.folderBrowserDialogXrefDir.SelectedPath;
                // force validation
                this.textBoxOutputFolder.Focus();
                this.buttonSelectOutputFolder.Focus();
            }
        }

        /// <summary>
        /// Builds the Platter Summary and Platter Detail reports.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPlatterReports_Click(object sender, EventArgs e)
        {
            string buttonPlatterReportText = "";
            if (buttonPlatterReports.Text == CANCEL)
            {
                if (this.ConfirmCancel())
                {
                    this.jukebox.CanRun = false;
                }
            }
            else
            {
                if (Directory.Exists(this.textBoxOutputFolder.Text))
                {
                    this.jukebox.OutputFolder = this.textBoxOutputFolder.Text;
                    this.jukebox.CanRun = true;
                    this.jukebox.FileTimeDelay = (int)this.numericUpDownFileTimeDelay.Value;
                    // re-configure the UI
                    buttonPlatterReportText = buttonPlatterReports.Text;
                    buttonPlatterReports.Text = CANCEL;
                    this.exitToolStripMenuItem.Enabled = false;
                    this.buttonSelectOutputFolder.Enabled = false;
                    this.comboBoxNetworkShare.Enabled = false;
                    this.exitMessage = null; // null means no dialog will be displayed at the end of processing by default
                    this.exitMessageTitle = null;

                    try
                    {
                        if (this.comboBoxNetworkShare.SelectedValue.ToString() == JukeboxHelper.ALL_NETWORK_SHARES)
                        {
                            // do all shares
                            if (this.ConfirmReplace(this.textBoxOutputFolder.Text) == true)
                            {
                                this.Logger().Info(this.Info(DateTime.Now.ToString() + ": Building Platter Reports"));
                                this.exitMessageTitle = "Platter Summary Report";
                                this.jukebox.PlatterSummaryReport();
                                if (this.jukebox.CanRun)
                                {
                                    foreach (KeyValuePair<String, NetworkShare> keyValue in dexShares)
                                    {
                                        this.jukebox.WritePlatterDetailReportsForNetworkShare(keyValue.Value);
                                        this.toolStripStatusLabelCurrentFile.Text = "";
                                        if (this.jukebox.CanRun == false)
                                        {
                                            break;
                                        }
                                    }
                                }
                                this.exitMessageTitle = "Platter Reports";
                                this.exitMessage = (this.jukebox.CanRun ? "Platter Reports are complete." : "Canceled by operatior.");
                                this.Logger().Info(this.Info(DateTime.Now.ToString() + ": " + exitMessage));
                            }
                        }
                        else // do selected share
                        {
                            // initialize the jukebox helper class

                            NetworkShare selectedShare = this.dexShares[this.comboBoxNetworkShare.SelectedValue.ToString()];
                            if (this.jukebox.IsNetworkSharePlatterReportsInFolder(selectedShare, this.textBoxOutputFolder.Text) == false)
                            {
                                this.Logger().Info(this.Info(DateTime.Now.ToString() + ": Building Platter Reports"));
                                this.exitMessageTitle = "Platter Summary Report";
                                this.jukebox.PlatterSummaryReport();
                                if (this.jukebox.CanRun)
                                {
                                    exitMessageTitle = "Platter Detail Reports";
                                    this.jukebox.WritePlatterDetailReportsForNetworkShare(selectedShare);
                                    this.toolStripStatusLabelCurrentFile.Text = "";
                                }
                                this.exitMessageTitle = "Platter Reports";
                                this.exitMessage = (this.jukebox.CanRun ? "Platter Reports are complete." : "Canceled by operatior.");
                                this.Logger().Info(this.Info(DateTime.Now.ToString() + ": " + exitMessage));
                            }
                            else
                            {
                                MessageBox.Show("Platter reports for network share " + selectedShare.ShareName + " already exist in " +
                                    this.jukebox.OutputFolder + ". Please select a new output folder for reports.", "Network Share Platter Reports already exist");
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        this.exitMessage = "Abort due to exception: " + ex.Message;
                        this.Logger().Info(this.Info(exitMessage));
                    }
                    finally
                    {
                        //this.ControlBox = true;
                        // cleanup - this closes the current platter detail report file
                        this.jukebox.PlatterReportCleanup();
                        // re-configure the UI
                        this.buttonSelectOutputFolder.Enabled = true;
                        this.comboBoxNetworkShare.Enabled = true;
                        this.exitToolStripMenuItem.Enabled = true;
                        buttonPlatterReports.Text = buttonPlatterReportText;
                        this.jukebox.CanRun = false;
                        this.toolStripStatusLabelCurrentFile.Text = "";
                        // report status to user
                        if (this.exitMessage != null && this.exitMessageTitle != null)
                        {
                            MessageBox.Show(exitMessage, exitMessageTitle, MessageBoxButtons.OK, MessageBoxIcon.None);
                        }
                    }
                }
                else // no output folder specified
                {
                    MessageBox.Show("You must select an output folder.");
                }
            }
        }

        /// <summary>
        /// Close MainForm which terminates the Dexter application.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Validates the contents of textBoxXrefDir which contains the report output directory to 
        /// ensure that reports will not be written to the DEX drive, or into the root directory of any drive.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void textBoxOutputFolder_Validating(object sender, CancelEventArgs e)
        {
            if (textBoxOutputFolder.Text.Length > 0)
            {
                if (Directory.GetDirectoryRoot(textBoxOutputFolder.Text).ToUpper() == textBoxOutputFolder.Text.ToUpper())
                {
                    MessageBox.Show("Dexter will not write report files into the root folder - please select a subfolder",
                        "Illegal Folder Selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    textBoxOutputFolder.Text = "";
                    e.Cancel = true;
                }
                else if (this.jukebox.IsDexDrive(textBoxOutputFolder.Text))
                {
                    MessageBox.Show("Dexter will not write report files on a DEX drive - please select a diffferent Drive",
                        "Disk Extender Drive Selected", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    textBoxOutputFolder.Text = "";
                    e.Cancel = true;
                }
            }
        }

        /// <summary>
        /// Display the Dexter About Box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAbout_Click(object sender, EventArgs e)
        {
            this.AboutDexter();
        }

        /// <summary>
        /// This event updates the jukebox helper's FileTimeDelay property so that JukeboxHelper.WritePlatterDetailReportsForFolder
        /// can sleep between each file. This allows Dexter to slow down and consume less system resources.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void numericUpDownFileTimeDelay_ValueChanged(object sender, EventArgs e)
        {
            if (this.jukebox != null)
            {
                this.jukebox.FileTimeDelay = (int)this.numericUpDownFileTimeDelay.Value;
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void aboutDexterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.AboutDexter();
        }

        #endregion

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.buttonPlatterReports.Text == CANCEL)
            {
                e.Cancel = true;
            }
        }

    }
}