{*
 Contains classes and types for making RPC calls to the VistA database
 }
unit cMagKatBroker;

interface

uses
  Trpcb, sysutils, rpcconf1, controls, TLoggerUnit, classes, cMagKatUtils;

type

  /// Defines the possible results from a login
  TMagKatBrokerLoginResult = (MagKatLoginOK, MagKatLoginError,
    MagKatLoginContextFailed, MagKatLoginCancelled);

  {*
    Makes all calls to the VistA system
  }
  TMagKatBroker = class
  private
    /// The broker instance to use to communicate with the VistA database
    FBroker : TRPCBroker;
    /// general information logger
    FLogger : TLogger;
    /// Error logget
    FErrorLogger : TLogger;

    function getConnected() : boolean;
    function getUserFullName() : String;
  public
    constructor Create(Logger : TLogger; ErrorLogger : TLogger);
    destructor Destroy;  override;

    function login(var server, port : string; var ErrMsg : String) : TMagKatBrokerLoginResult;
    function Logout() : boolean;

    function findImageTextFile(ImageName : String; var output : String) : boolean;

    function scanMagScanImageTextFiles(currentIen : String; direction : string; var output : string) : boolean;
    function getSiteParameterCredentials(var username, password, PrimarySiteStationNumber : String) : boolean;
    function getNetworkLocations(var Shares : TStrings) : boolean;
    function updateVistATxtFields(ImageIEN, seriesUid, seriesnumber,
      imageNumber, imageUid, studyUid, documentStamp : string; var msg : String) : boolean;
    function initMagScanImageTextFiles(var output : string) : boolean;
    function restartMagScanImageTextFiles(var output : string) : boolean;
    procedure RPMaggInstall(var Fstat: boolean; var Flist: tstringlist);
    procedure RPMagVersionStatus(var status: string;  version: string);
    function getUserKeys(var keys : TStringlist) : boolean;
    function getRequiredContext() : string;

    /// True if the broker is connected, false otherwise
    property Connected : boolean read getConnected;
    /// The connected users full name
    property UserFullName : String read getUserFullName;

end;

const
  /// Required context for running MagKat
  MAG_KAT_CONTEXT = 'MAGKAT';
  /// RPC to scan for more TXT files
  RPC_MAG_SCAN_IMAGE_TEXT_FILES = 'MAG SCAN IMAGE TEXT FILES';
  /// RPC to get list of installed Imaging versions
  RPC_MAG_INSTALL = 'MAGG INSTALL';
  /// RPC to get information from Imaging Site Parameters file
  RPC_MAGGUSER2 = 'MAGGUSER2';
  /// RPC to get network locations
  RPC_MAG_GET_NETLOC = 'MAG GET NETLOC';
  /// RPC to store updated details from TXT file
  RPC_MAG_STORE_TEXT_FILE_DETAILS = 'MAG STORE TEXT FILE DETAILS';
  /// RPC get get version status
  RPC_MAG4_VERSION_STATUS = 'MAG4 VERSION STATUS';
  /// RPC to get user security keys
  RPC_MAGGUSERKEYS = 'MAGGUSERKEYS';
  /// RPC to get TXT file details from filename
  RPC_MAG_FIND_IMAGE_TEXT_FILE = 'MAG FIND IMAGE TEXT FILE';

implementation

{*
  Creates a new TMagKatBroker
  @param Logger The default logger
  @param ErrorLogger Logger to write errors to
 }
constructor TMagKatBroker.Create(Logger : TLogger; ErrorLogger : TLogger);
begin
  inherited create();
  FBroker := TRpcBroker.Create(nil);
  FLogger := Logger;
  FErrorLogger := ErrorLogger;
end;

{*
 Destructor destroys all allocated variables
}
destructor TMagKatBroker.Destroy;
begin
  inherited;
  if FBroker <> nil then
  begin
    FBroker.Connected := false;
    FreeAndNil(FBroker);
  end;
end;

{*
 Logs into the VistA system
 @param server The server to connect to (can be empty)
 @param port the port to connect to (can be empty)
 @return Result to indicate the status, ok if login is ok, broker error if there
 is a broker error or context failed if the user doesn't have the required context
 }
function TMagKatBroker.login(var server, port : string; var ErrMsg : String) : TMagKatBrokerLoginResult;
begin
  result := MagKatLoginError;
  flogger.Info('Logging into site [' + server + ', ' + port + ']');
  try
    // if a value is specified from the user, connect to that server and port
    // otherwise get the value from the broker
    if ((server <> '') and (port <> ''))
      or (GetServerInfo(server, port) = mrok) then
    begin
      Flogger.Info('Connecting to site [' + server + ', ' + port + ']');
      fbroker.Server := server;
      fbroker.ListenerPort := strtoint(port);
      fbroker.Connected := true;
      if fbroker.Connected then
      begin
        Flogger.Info('Connected to VistA');
        Flogger.Info('Creating context [' + MAG_KAT_CONTEXT + ']');
        if not fbroker.CreateContext(MAG_KAT_CONTEXT) then
        begin
          ErrMsg := 'Unable to create context [' + MAG_KAT_CONTEXT +
            '], disconnecting broker';
          FLogger.Error(ErrMsg);
          FBroker.Connected := false;
          result := MagKatLoginContextFailed;
        end
        else
        begin
          result := MagKatLoginOK;
        end;
      end;
    end
    else
    begin
      FLogger.Info('Connection to VistA cancelled by user');
      result := MagKatLoginCancelled;
    end;
  except
    on E : Exception do
    begin
      result := MagKatLoginError;
      flogger.error('TMagKatBroker.login, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.login, ' + e.Message);
      ErrMsg := e.Message;
    end;
  end;
end;

{*
 Disconnects the broker connection
 @return true if the disconnect was ok, false if there was an error
 }
function TMagKatBroker.Logout() : boolean;
begin
  try
    if FBroker <> nil then
    begin
      FBroker.Connected := false;
    end;
    result := not fbroker.Connected;
  except
    on E : exception do
    begin
      result := false;
      flogger.Error('TMagKatBroker.Logout, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.Logout, ' + e.Message);
    end;
  end;
end;

{*
 Returns the connected status
 @return true if connected, false if not connected
 }
function TMagKatBroker.getConnected() : boolean;
begin
  if FBroker = nil then
    result := false
  else
    result := FBroker.Connected;
end;

{*
  Get the next image information based on the current Ien and the direction
  @param currentIen the IEN of the current image, will return the next TXT file
  @param direction The direction to scan (1 forwards, -1 backwards)
  @param output The output response
  @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.scanMagScanImageTextFiles(currentIen : String; direction : string; var output : string) : boolean;
begin
  result := false;
  try
    FBroker.RemoteProcedure := RPC_MAG_SCAN_IMAGE_TEXT_FILES;
    FBroker.Param[0].PType := Literal;
    FBroker.Param[0].Value := 'Scan';
    FBroker.Param[1].PType := Literal;
    FBroker.Param[1].Value := currentIen;
    FBroker.Param[2].PType := Literal;
    FBroker.Param[2].Value := direction;
    output := FBroker.strCall;
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('TMagKatBroker.scanMagScanImageTextFiles, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.scanMagScanImageTextFiles, ' + e.Message);
    end;
  end;
end;

{*
 initialize the scan of the TXT files (get the last status of the updates)
 @param output The output response from the init call
 @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.initMagScanImageTextFiles(var output : string) : boolean;
begin
  result := false;
  try
    FBroker.RemoteProcedure := RPC_MAG_SCAN_IMAGE_TEXT_FILES;
    FBroker.Param[0].PType := Literal;
    FBroker.Param[0].Value := 'Init';
    output := FBroker.strCall;
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('TMagKatBroker.initMagScanImageTextFiles, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.initMagScanImageTextFiles, ' + e.Message);
      result := false;
    end;
  end;
end;

{*
  Retrieve the default credentials from the Imaging Site Parameters file
  @param username the username from VistA
  @param password the Password from VistA
  @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.getSiteParameterCredentials(var username,
  password, PrimarySiteStationNumber : String) : boolean;
var
  rList : tStrings;
begin
  try
    rlist := TStringList.Create();
    FBroker.RemoteProcedure := RPC_MAGGUSER2;
    FBroker.Param[0].PType := literal;
    FBroker.Param[0].Value := 'MagKat';

    fbroker.lstCall(rlist);

    if rlist.Count <= 2 then
    begin
      FLogger.Error('Incorrect data recieved from MAGGUSER2 rpc');
      if rlist.Count > 0 then
        FLogger.Error('TMagKatBroker.getSiteParameterCredentials, MAGGUSER2 response [' + rlist[0] + ']');
      result := false;
      exit;
    end;
    username := MagPiece(rlist[2], '^', 1);
    password := MagPiece(rlist[2], '^', 2);//Copy(rlist[2], length(username), 1000);
    PrimarySiteStationNumber := rlist[12];
    result := true;
    if rList <> nil then
      FreeAndNil(rList);
  except
    on E :Exception do
    begin
      result := false;
      FLogger.error('TMagKatBroker.getSiteParameterCredentials, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.getSiteParameterCredentials, ' + e.Message);
    end;
  end;
end;

{*
  Retrieves the network locations from VistA
  @param Shares The shares from the VistA database
  @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.getNetworkLocations(var Shares : TStrings) : boolean;
begin
  try
    FBroker.PARAM[0].VALUE := 'ALL';
    FBroker.PARAM[0].PTYPE := LITERAL;
    Fbroker.remoteprocedure := RPC_MAG_GET_NETLOC;

    FBroker.lstCall(Shares);

    if (magpiece(Shares[0], '^', 1) = '0') then
      begin
        FLogger.Error(magpiece(Shares[0], '^', 2));
        exit;
      end;
    if (Shares.count = 0) then
      begin
        FLogger.Error('TMagKatBroker.getNetworkLocations, Error accessing Network Locations');
        FErrorLogger.Error('TMagKatBroker.getNetworkLocations, Error accessing Network Locations');
        exit;
      end;
    Shares.delete(0);
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('TMagKatBroker.getNetworkLocations, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.getNetworkLocations, ' + e.Message);
      result := false;
    end;
  end;
end;

{*
  Updates the TXT file in VistA with the new values
  @ImageIEN the IEN of the TXT file to update
  @seriesUid The new uid from the TXT file
  @seriesNumber the new series number from the TXT file
  @imageNumber the new image number from the TXT file
  @imageUid the new image Uid from the TXT file
  @studyUid the new study UID from the TXT file
  @documentStamp the new document stamp from the TXT file
  @param msg the response from the RPC
  @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.updateVistATxtFields(ImageIEN, seriesUid, seriesnumber,
  imageNumber, imageUid, studyUid, documentStamp : string; var msg : String) : boolean;
begin
  try
    FBroker.Param[0].PType := LITERAL;
    FBroker.Param[0].Value := ImageIEN;
    FBroker.Param[1].PType := LITERAL;
    FBroker.Param[1].Value := seriesUid;
    FBroker.Param[2].PType := LITERAL;
    FBroker.Param[2].Value := seriesnumber;
    FBroker.Param[3].PType := LITERAL;
    FBroker.Param[3].Value := imageNumber;
    FBroker.Param[4].PType := LITERAL;
    FBroker.Param[4].Value := imageUid;
    FBroker.Param[5].PType := LITERAL;
    FBroker.Param[5].Value := studyUid;
    FBroker.Param[6].PType := LITERAL;
    FBroker.Param[6].Value := documentStamp;

    FBroker.RemoteProcedure := RPC_MAG_STORE_TEXT_FILE_DETAILS;
    msg := FBroker.strCall;
    if MagPiece(msg, ',', 1) = '0' then
    begin
      FLogger.info('TMagKatBroker.updateVistATxtFields, [' + imageIen + '] updated result [' + msg + ']');
    end
    else
    begin
      FLogger.info('TMagKatBroker.updateVistATxtFields, [' + imageIen + '] not updated result [' + msg + ']');
    end;
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('TMagKatBroker.updateVistATxtFields, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.updateVistATxtFields, ' + e.Message);
      result := false;
    end;
  end;
end;

{*
  Restarts the scanning process
  @param output The response from the RPC call
  @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.restartMagScanImageTextFiles(var output : string) : boolean;
begin
  result := false;
  try
    FBroker.RemoteProcedure := RPC_MAG_SCAN_IMAGE_TEXT_FILES;
    FBroker.Param[0].PType := Literal;
    FBroker.Param[0].Value := 'Restart';
    output := FBroker.strCall;
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('TMagKatBroker.restartMagScanImageTextFiles, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.restartMagScanImageTextFiles, ' + e.Message);
      result := false;
    end;
  end;
end;

{*
  Retrieves the list of installed versions of VistA Imaging
  @param Fstat the status of the RPC call
  @param Flist the list of installed versions of Imaging
 }
procedure TMagKatBroker.RPMaggInstall(var Fstat: boolean; var Flist: tstringlist);
 var I: integer;
begin
if (FBroker = nil) or (not Fbroker.Connected) then
 begin
 Flist.add('0^No connection to the Server');
 exit;
 end;
  Flist.clear;
  Fstat := false;
  Fbroker.remoteprocedure := RPC_MAG_INSTALL;
  try
    Fbroker.lstCall(Flist);
    if (Flist.Count = 0) or (magpiece(Flist[0], '^', 1) = '0') then exit;
    Fstat := true;
    if maglength(Flist[0],'~') = 1 then  // This is old M code, not returning '~' codes for columns;
       Flist[0] := 'Server Versions  ^Install Date     ~S1'; // this sort code 'S1' says that this column is Dates.
  except
    on E: EXCEPTION do
      begin
        Flist.Insert(0, '0^' + magpiece(e.message, #$A, 2));
        FErrorLogger.Error('TMagKatBroker.restartMagScanImageTextFiles, ' + e.Message);
        FLogger.Error('TMagKatBroker.restartMagScanImageTextFiles, ' + e.Message);
      end;
  end;
end;

{*
  Retrieves the version status from VistA
  @param status the status response message
  @param version The version
 }
procedure TMagKatBroker.RPMagVersionStatus(var status: string;  version: string);
begin
if (FBroker = nil) or (not Fbroker.Connected) then
 begin
 status := '1^Status is unavailable.';
 exit;
 end;
  Fbroker.remoteprocedure := RPC_MAG4_VERSION_STATUS;
  FBroker.Param[0].PType := literal;
  FBroker.Param[0].Value := Version;

  try
   status :=  FBroker.strcall;
    if (status = '') then
      begin
        status := '1^Error trying to determine status';
        FLogger.Error('Error Checking Status of Version: ' + version);
        FErrorLogger.Error('Error Checking Status of Version: ' + version);
        exit;
      end;         
  except
    on e: exception do
      begin
        status := '1^Error trying to determine status';
        FErrorLogger.Error('TMagKatBroker.RPMagVersionStatus, ' + e.Message);
        FLogger.Error('TMagKatBroker.RPMagVersionStatus, ' + e.Message);
      end;
  end;
end;

{*
 Gets the keys the user has on the VistA system
 @param keys The list of keys the user has assigned
 @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.getUserKeys(var keys : TStringlist) : boolean;
begin
  result := false;
  try
    FBroker.RemoteProcedure := RPC_MAGGUSERKEYS;
    FBroker.lstCall(keys);
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('TMagKatBroker.getUserKeys, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.getUserKeys, ' + e.Message);
      result := false;
    end;
  end;
end;

{*
  Return the VistA context required to run MagKat
  @return The context
 }
function TMagKatBroker.getRequiredContext() : string;
begin
  result := MAG_KAT_CONTEXT;
end;

{*
  Find the TXT file associated with an image filename
  @param ImageName The name of the file (not full path)
  @param output The response from the RPC call
  @return True if the RPC ran successfully, false if there was an error
 }
function TMagKatBroker.findImageTextFile(ImageName : String; var output : String) : boolean;
begin
  result := false;
  try
    FBroker.RemoteProcedure := RPC_MAG_FIND_IMAGE_TEXT_FILE;
    FBroker.Param[0].PType := literal;
    FBroker.Param[0].Value := ImageName;
    output := FBroker.strCall;
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('TMagKatBroker.findImageTextFile, ' + e.Message);
      FErrorLogger.Error('TMagKatBroker.findImageTextFile, ' + e.Message);
      result := false;
    end;
  end;
end;

{*
  Returns the connected users full name. If the broker is nil or not connected
  then this returns an empty string
  @return The connected users name
 }
function TMagKatBroker.getUserFullName() : String;
begin
  result := '';
  if FBroker = nil then
    exit;
  if not FBroker.Connected then
    exit;
  result := FBroker.User.Name;
end;

end.
