{*
 Contains classes and types for processing the TXT files for the MagKat utility
 }
unit cMagKatEngine;

interface

  uses cMagKatBroker, TLoggerUnit, sysutils, cMagKatUtils, cMagSecurity,
  classes, windows, cMagKatJukeboxFileEngine, cMagKatPlatter, hash;

type

  /// the direction of the processing for Magnetic processing
  TMagKatProcessDirection = (MagKatProcessForwards, MagKatProcessBackwards);
  /// The possible locations of the TXT files
  TMagKatTXTFileLocation = (MagKatMagnetic, MagKatJukebox, MagKatOffline,
    MagKatArchive, MagKatInvalid, MagKatAll);

  /// The types of actions that are taken on a platter text file
  TMagKatFileProcessedAction = (MagKatProcessedSuccessfully,
    MagKatPreviouslyProcessed, MagKatProcessError, MagKatFileIOError,
    MagKatSkippingFile, MagKatNA);

  {*
    Event for when a message is to be sent from the engine to the parent
    @param Sender the object which sent this event
    @param msg The message from the engine
    }
  TMagKatEngineProcessMessageEvent = procedure(Sender: TObject; msg : String) of object;
  {*
    Event for when a file has been processed
    @param Sender the object which sent this event
    @IEN The IEN of the TXT file that has been processed
    @FileLocation The location of the file that has been processed
    }
  TMagKatEngineFileProcessedEvent = procedure(Sender : TObject; IEN : String; FileLocation : TMagKatTXTFileLocation) of object;
  {*
    Event for when a new IEN is being processed
    @param Sender the object which sent this event
    @param IEN the IEN of the TXT file currently processing
   }
  TMagKatEngineCurrentIENChangedEvent = procedure(Sender : TObject; IEN : String) of object;
  {*
    Event for when processing stops because the user pressed a stop button
    @param Sender the object which sent this event
    @param LastIENProcessed The IEN of the TXT file that was last processed before the stop event occured
    }
  TMagKatEngineProcessingStoppedEvent = procedure(Sender : TObject; LastIENProcessed : string) of object;
  {*
    Event for when the rate of processing has changed
    @param Sender the object which sent this event
    @param currentRate the Current rate of processing
    }
  TMagKatEngineUpdateCurrentRateEvent = procedure(Sender : TObject; currentRate : String) of object;
  {*
    Event for when processing has been completed
    @param Sender the object which sent this event
    @param LastIENProcessed The last IEN that was processed before the engine completed
    @param Msg A message to display
    }
  TMagKatEngineProcessingCompletedEvent = procedure(Sender : TObject; LastIENProcessed : String; Msg : string) of object;
  {*
    Event for when a platter has changed the number of images processed, total, or failed
    @param Sender the object which sent this event
    @param processed The number of TXT files processed
    @param failed The number of TXT files failed
    @param total The total number of TXT files on the platter
    }
  TMagKatEnginePlatterProcessingUpdateEvent = procedure(Sender : TObject;
    Platter : TMagKatPlatter; processed, failed, total : integer) of object;

  {*
   Event for when the recent error count event occurs - to display a new recent
   error count
   @param Sender the object which sent this event
   @param recentFailedCount The message containing the recent number of images that had an error
  }
  TMagKatEngineUpdateRecentErrorCountEvent = procedure(Sender : TObject; recentFailedCount : string) of object;

  {*
    Keeps track of the progress of the processing.  Contains counts of the
    status of the work that has been done.
  }
  TMagKatProcessingProgress = class
  private
    /// The number of files processed
    FProcessedFiles : integer;
    /// The number of files with errors
    FErrorFiles : integer;
    /// the number of files skipped because they are on jukeboxes
    FJukeboxSkippedFiles : integer;
    /// The number of files skipped for being offline
    FOfflineSkippedFiles : integer;
    /// The number of files skipped for being on unknown shares
    FUnknownShareSkippedFiles : integer;
    /// The number of files which have previously been processed
    FAlreadyProcessedFiles : integer;
    /// The number of files with I/O errors
    FIOErrors : integer;
    /// The number of platters processed
    FPlattersProcessed : integer;
    /// The number of file skipped for integrity problems
    FSkippedFiles : integer;
  public
    constructor Create();
    procedure Reset();

    /// The number of files processed
    property ProcessedFiles : integer read FProcessedFiles ;
    /// The number of files with errors
    property ErrorFiles : integer read FErrorFiles;
    /// the number of files skipped because they are on jukeboxes
    property JukeboxSkippedFiles : integer read FJukeboxSkippedFiles;
    /// The number of files skipped for being offline
    property OfflineSkippedFiles : integer read FOfflineSkippedFiles;
    /// The number of files skipped for being on unknown shares
    property UnknownShareSkippedFiles : integer read FUnknownShareSkippedFiles;
    /// The number of files which have previously been processed
    property AlreadyProcessedFiles : integer read FAlreadyProcessedFiles;
    /// The number of files with I/O errors
    property IOErrors : integer read FIOErrors;
    /// The number of platters processed
    property PlattersProcessed : integer read FPlattersProcessed;
    /// The number of file skipped for integrity problems
    property SkippedFiles : integer read FSkippedFiles;

  end;


  {*
  Main engine for processing TEXT files for MagKat utility.
  @author Julian Werfel
  }
  TMagKatEngine = class
  private
    /// RPC broker to use to communicate with the database
    FBroker : TMagKatBroker;
    /// Logger for general output messages
    FLogger : TLogger;
    /// Logger for completed files
    FCompletedLogger : TLogger;
    /// Logger for error messages
    FErrorLogger : TLogger;
    /// Security tool for connecting to network shares
    FMagSecurity : TMag4Security;
    /// Jukebox file engine to read and write files that contain jukebox platter information
    FMagKatJukeboxFileEngine : TMagKatJukeboxFileEngine;

    /// Event for processing messages
    MagKatEnginePRocessMessageEvent : TMagKatEngineProcessMessageEvent;
    /// Event for when a file has been processed
    MagKatEngineFileProcessedEvent : TMagKatEngineFileProcessedEvent;
    /// Event for when a new IEN is being processed
    MagKatEngineCurrentIENChangedEvent : TMagKatEngineCurrentIENChangedEvent;
    /// Event for when processing has stopped
    MagKatEngineProcessingStoppedEvent : TMagKatEngineProcessingStoppedEvent;
    /// Event for when the rate of processing is updated
    MagKatEngineUpdateCurrentRateEvent : TMagKatEngineUpdateCurrentRateEvent;
    /// Event for when processing has completed
    MagKatEngineProcessingCompletedEvent : TMagKatEngineProcessingCompletedEvent;
    /// Event for when a platter has an item changed in processing
    MagKatEnginePlatterProcessingUpdateEvent : TMagKatEnginePlatterProcessingUpdateEvent;
    /// Event for when a platter has updated its status
    MagKatPlatterStatusUpdateUpdateEvent : TMagKatPlatterStatusUpdateUpdateEvent;
    /// Event for when the recent error count event occurs
    MagKatEngineUpdateRecentErrorCountEvent : TMagKatEngineUpdateRecentErrorCountEvent;

    /// Username for shares that do not have a username specified
    FUsername : String;
    /// Password for shares that do not have a password specified
    FPassword : String;
    /// True if processing has been paused, false if not paused
    FProcessingPaused : boolean;
    /// The IEN of the last TXT file processed
    FLastProcessedIen : String;
    /// The amount of time (in ms) to delay between processing TXT files
    FProcessingDelay  : integer;
    /// The direction of processing
    FProcessDirection  : TMagKatProcessDirection;
    /// The first IEN in the range of IEN's successfully processed currently
    FStartCompletedIEN : String;
    /// The time the last updated rate was calculated
    FLastUpdatedRateTime : int64;
    /// The image processed count
    FImageProcessedCount : integer;
    /// Holds the current progress of the processing currently occuring
    FCurrentProcessingProgress : TMagKatProcessingProgress;
    /// The number of images that failed to process in the current run (for recent error count)
    FRecentFailedImageCount : integer;
    /// determines if the currently processing a platter
    FIsProcessingPlatters : boolean;

    function ProcessTXTFile(IEN : String; Direction : TMagKatProcessDirection; LocationToProcess : TMagKatTXTFileLocation) : string;
    function readTXTFileAndUpdateVista(IEN : String; Filename : String;
      var errMsg : String; PlatterSerial : String = '') : TMagKatFileProcessedAction;
    procedure LogProcessMessage(Msg : string);
    function getTxtFileLocation(VistaLocation : String) : TMagKatTXTFileLocation;

    procedure LogCompletedImages(StartIEN, FinishIEN : String);
    procedure CalculateCurrentProcessingRate();
    function ProcessPlatterTXTFile(FullFilename : String;
      PlatterSerial : String; var errorMsg : String) : TMagKatFileProcessedAction;
    function LoadPlatterFiles(Platter : TMagKatPlatter) : TStrings;

    function MagKatFileProcessedActionToString(Action : TMagKatFileProcessedAction) : string;

    function isShareKnown(Filename : String) : boolean;

  public
    constructor Create(Broker : TMagKatBroker; Logger : TLogger;
      CompleteLogger : TLogger; ErrorLogger : TLogger;
      jukeboxFileEngine : TMagKatJukeboxFileEngine);
    destructor Destroy; override;

    function ProcessRange(startIen, endIen : integer; FileLocation : TMagKatTXTFileLocation) : boolean;
    procedure ProcessSequentially(StartIen : string;
      Direction : TMagKatProcessDirection; FileLocation : TMagKatTXTFileLocation);
    procedure SetImageShareCredentials(Username, Password : String; Shares : TStrings);
    procedure ProcessPlatter(platter: TMagKatPlatter;
      AlertDoneOnComplete : boolean = true);
    procedure ProcessPlatterList(PlatterList : TList);

    function stopProcessing() : string;
    procedure ClearImageShareCredentials();
    function getCurrentProcessingProgressDetails() : string;

    property Username : String read FUsername write FUsername;
    property Password : String read FPassword write FPassword;

  published
    /// Event occurs when a message is to be displayed to the user
    property OnMagKatEngineProcessMessage : TMagKatEngineProcessMessageEvent read MagKatEnginePRocessMessageEvent write MagKatEnginePRocessMessageEvent;
    /// event occurs when a file has been processed
    property OnMagKatEngineFileProcessed : TMagKatEngineFileProcessedEvent read MagKatEngineFileProcessedEvent write MagKatEngineFileProcessedEvent;
    property OnMagKatEngineCurrentIENChanged : TMagKatEngineCurrentIENChangedEvent read MagKatEngineCurrentIENChangedEvent write MagKatEngineCurrentIENChangedEvent;
    property ProcessingDelay : integer read FProcessingDelay write FProcessingDelay;
    property OnMagKatEngineProcessingStopped : TMagKatEngineProcessingStoppedEvent read MagKatEngineProcessingStoppedEvent write MagKatEngineProcessingStoppedEvent;
    property OnMagKatEngineUpdateCurrentRateEvent : TMagKatEngineUpdateCurrentRateEvent read MagKatEngineUpdateCurrentRateEvent write MagKatEngineUpdateCurrentRateEvent;
    property OnMagKatEngineProcessingCompletedEvent : TMagKatEngineProcessingCompletedEvent read MagKatEngineProcessingCompletedEvent write MagKatEngineProcessingCompletedEvent;
    property OnMagKatEnginePlatterProcessingUpdateEvent : TMagKatEnginePlatterProcessingUpdateEvent read MagKatEnginePlatterProcessingUpdateEvent write MagKatEnginePlatterProcessingUpdateEvent;
    property OnMagKatPlatterStatusUpdateUpdateEvent : TMagKatPlatterStatusUpdateUpdateEvent read MagKatPlatterStatusUpdateUpdateEvent write MagKatPlatterStatusUpdateUpdateEvent;
    property OnMagKatEngineUpdateRecentErrorCountEvent : TMagKatEngineUpdateRecentErrorCountEvent read MagKatEngineUpdateRecentErrorCountEvent write MagKatEngineUpdateRecentErrorCountEvent;
  end;

const
  /// The number of I/O errors to allow before platter processing stops
  IO_ERROR_COUNT_MAX = 2;

implementation

uses StrUtils;

{*
  Constructor that requires the broker connection, loggers and the jukebox
  file processing engine
  @param Broker Connected broker used to make RPC calls with
  @param Logger Default general logger for all output messages
  @param CompleteLogger Logger for completed files
  @param ErrorLogger Logger for error messages
  @param jukeboxFileEngine Jukebox platter file engine
  }
constructor TMagKatEngine.Create(Broker : TMagKatBroker; Logger : TLogger;
  CompleteLogger : TLogger; ErrorLogger : TLogger;
  jukeboxFileEngine : TMagKatJukeboxFileEngine);
begin
  inherited Create;
  FBroker := Broker;
  FLogger := Logger;
  FErrorLogger := ErrorLogger;
  FCompletedLogger := CompleteLogger;
  FMagKatJukeboxFileEngine := jukeboxFileEngine;
  FMagSecurity := TMag4Security.create(nil);
  FProcessingDelay := 0;
  FProcessDirection := MagKatProcessBackwards;
end;

{*
  Destructor responsible for destroying all objects created by the engine
 }
destructor TMagKatEngine.Destroy;
var
  msg : string;
begin
  inherited;
  if FMagSecurity <> nil then
  begin
    FMagSecurity.MagCloseSecurity(msg);
    FreeAndNil(FMagSecurity);
  end;
  if FCurrentProcessingProgress <> nil then
    FreeAndNil(FCurrentProcessingProgress);
end;

{*
 Processes a list of platters
 @param PlatterList the list of platters to process, should be list of TMagKatPlatter objects
 }
procedure TMagKatEngine.ProcessPlatterList(PlatterList : TList);
var
  i : integer;
  platter : TMagKatPlatter;
begin
  try
    if FCurrentProcessingProgress = nil then
      FCurrentProcessingProgress := TMagKatProcessingProgress.Create()
    else
      FCurrentProcessingProgress.Reset();
    FIsProcessingPlatters := true;
    for i := 0 to PlatterList.Count - 1 do
    begin
      platter := PlatterList.Items[i];
      LogProcessMessage('Processing platter "' + platter.PlatterDisplayName + '"');
//      FLogger.Info('Processing platter "' + platter.PlatterDisplayName + '"');   // redundant
      ProcessPlatter(platter, false);
      if FProcessingPaused then
        exit;
      // test handling user pressing the pause button
      LogProcessMessage('Platter "' + platter.PlatterDisplayName + '" has completed processing');
//      FLogger.Info('Platter "' + platter.PlatterDisplayName + '" has completed processing'); // redundant
    end;
  finally
    if assigned(MagKatEngineProcessingCompletedEvent) then
    begin
      if FProcessingPaused then
        MagKatEngineProcessingCompletedEvent(self, FLastProcessedIen,
          'Platter processing paused by user')
      else
        MagKatEngineProcessingCompletedEvent(self, FLastProcessedIen,
          'Completed processing images');
    end;
  end;
end;

{*
  Processes an individual platter.
  @param platter The platter to process
  @param AlertDoneOnComplete If true the engine will fire the complete event when it has finished
  processing the platter, if false it will not fire this event.
 }
procedure TMagKatEngine.ProcessPlatter(platter : TMagKatPlatter;
  AlertDoneOnComplete : boolean = true);
var
  isComplete : boolean;
  ioErrorStop : boolean;
  msg, txtLine : string;
  txtFilesProcessed, failedTxtFiles : integer;
  actualFilesProcessed : integer;
  ioErrors : integer;
  files : TStrings;
  i : integer;
  processTXTFileResult : TMagKatFileProcessedAction;
  errMsg : String;
begin
  FLogger.Info('Processing platter "' + platter.PlatterDisplayName + '"');
  ioErrorStop := false;
  FIsProcessingPlatters := true;
  if FCurrentProcessingProgress = nil then
    FCurrentProcessingProgress := TMagKatProcessingProgress.Create()
  else if AlertDoneOnComplete then
    FCurrentProcessingProgress.Reset();
  FCurrentProcessingProgress.FPlattersProcessed := FCurrentProcessingProgress.FPlattersProcessed + 1;
  txtFilesProcessed := 0;
  failedTxtFiles := 0;
  actualFilesProcessed := 0;
  FRecentFailedImageCount := 0;
  ioErrors := 0;
  files := nil;
  //DEX to JB1MO2-2B (74AC15BF42EA4F45).txt

  try
    if assigned(MagKatEngineUpdateCurrentRateEvent) then
      MagKatEngineUpdateCurrentRateEvent(self, 'Calculating processing rate...');
    if assigned(MagKatEngineUpdateRecentErrorCountEvent) then
      MagKatEngineUpdateRecentErrorCountEvent(self, 'Calculating processing rate...');
    FImageProcessedCount := 0;
    FLastUpdatedRateTime := GetTickCount;
    FProcessingPaused := false;
    FStartCompletedIEN := '';
    FLastProcessedIen := '';
    isComplete := false;
    try
      files := LoadPlatterFiles(platter);

      if assigned(MagKatEnginePlatterProcessingUpdateEvent) then
      begin
        MagKatEnginePlatterProcessingUpdateEvent(self,  platter,
          txtFilesProcessed, failedTxtFiles, platter.TxtFilesOnPlatter);
      end;            
      // JMW 11/18/2008 - if the files list is nil, then the platter file
      // does not exist for this platter, put platter into different group
      if files = nil then
      begin
        ioErrorStop := true;
        platter.PlatterStatus := OfflineError;
        FLogger.Error('Cannot process platter [' + platter.PlatterDisplayName +
          '] missing platter report file containing list of files on platter');
        FErrorLogger.Error('Cannot process platter [' + platter.PlatterDisplayName +
          '] missing platter report file containing list of files on platter');
        exit;
      end;
      for i := 0 to files.Count - 1 do
      begin
        txtLine := files[i];
        processTXTFileResult := ProcessPlatterTXTFile(txtline,
          platter.PlatterSerialNumber, errMsg);
        if processTXTFileResult = MagKatProcessedSuccessfully then
        begin
          txtFilesProcessed := txtFilesProcessed + 1;
          actualFilesProcessed := actualFilesProcessed + 1;
        end
        else if processTXTFileResult = MagKatProcessError then
        begin
          failedTxtFiles := failedTxtFiles + 1;
          FRecentFailedImageCount := FRecentFailedImageCount + 1;
        end
        else if processTXTFileResult = MagKatFileIOError then
        begin
          ioErrors := ioErrors + 1;
          FRecentFailedImageCount := FRecentFailedImageCount + 1;
        end
        else if (processTXTFileResult = MagKatSkippingFile) then
        begin
          // this occurs when the file is a duplicate, deleted, or IQ flag set
          // this is a valid reason to skip the file, not an error
          txtFilesProcessed := txtFilesProcessed + 1;
        end
        else if (processTXTFileResult = MagKatNA) then
        begin
          // these count as failed attempts, this is likely caused by running
          // MagKat on a database with platters that are not for this database
          // these should not be marked complete
          failedTxtFiles := failedTxtFiles + 1;
          FRecentFailedImageCount := FRecentFailedImageCount + 1;
        end
        else
        begin
          // this occurs when the TXT File was previously processed (MagKatPreviouslyProcessed)
          txtFilesProcessed := txtFilesProcessed + 1;
        end;
        if assigned(MagKatEnginePlatterProcessingUpdateEvent) then
        begin
          MagKatEnginePlatterProcessingUpdateEvent(self, platter,
            txtFilesProcessed, failedTxtFiles, platter.TxtFilesOnPlatter);
        end;
        files[i] := txtLine + #9 +
          MagKatFileProcessedActionToString(processTXTFileResult) + #9 + errMsg;
        if FProcessingPaused then
        begin
          FLogger.Info('Pausing Jukebox processing because pause button was pressed');
          exit;
        end;
        if ioErrors >  IO_ERROR_COUNT_MAX then
        begin
          ioErrorStop := true;
          platter.PlatterStatus := OnlineFailedProcess;
          FLogger.Info('Stopping platter processing because I/O Error count '+
            'has exceeded max allowed I/O errors [' + inttostr(IO_ERROR_COUNT_MAX) + ']');
          {
          if assigned(OnMagKatEngineProcessMessage) then
            OnMagKatEngineProcessMessage(self, 'Stopping platter processing because I/O Error count '+
              'has exceeded max allowed I/O errors [' + inttostr(IO_ERROR_COUNT_MAX) + ']');
          }
          exit;
        end;
      end;
      isComplete := true;
      // only update the status to processed if we actually did something
      // JMW 11/18/2008 - if the Txt files processed is > 0 then this platter
      // is processed, previously processed TXT files count as processed and
      // should move platter to completed status
      if txtFilesProcessed > 0 then
        platter.PlatterStatus := Processed;
      // JMW 11/18/2008 - if there are many failed Txt files (more than the
      // numebr of processed), the set the platter status to online failed
      if (failedTxtFiles > 0) and (failedTxtFiles > txtFilesProcessed) then
      begin
        platter.PlatterStatus := OnlineFailedProcess;
      end;
      // these two need to happen before the update event
      platter.FailedTxtFiles := failedTxtFiles;
      platter.TxtFilesProcessed := txtFilesProcessed; 
      // update the platter status in the platter file...
      if assigned(OnMagKatPlatterStatusUpdateUpdateEvent) then
        OnMagKatPlatterStatusUpdateUpdateEvent(self, platter);
    except
      on E : exception do
      begin
        FLogger.Error('Error processing files on platter [' + platter.PlatterSerialNumber + '], ' + e.Message);
        FErrorLogger.Error('Error processing files on platter [' + platter.PlatterSerialNumber + '], ' + e.Message);
      end;
    end;
  finally
    FLogger.Info('Platter "' + platter.PlatterDisplayName + '" has completed processing');
    FMagSecurity.MagCloseSecurity(msg);
    // if got here and the files are nil, that means there was no platter report
    // file for this platter, add an entry so it gets logged
    if files = nil then
    begin
      files := TStringList.Create();
      files.Add('' + #9 + '' + 'Missing Platter report file');
    end;
    FMagKatJukeboxFileEngine.writePlatterReport(platter, files);
    if files <> nil then
      FreeAndNil(files);
    FMagKatJukeboxFileEngine.writePlattertoFile(platter);
    platter.TxtFilesProcessed := txtFilesProcessed;
    platter.FailedTxtFiles := failedTxtFiles;
    platter.IOErrors := ioErrors;
    FMagKatJukeboxFileEngine.PlattersUpdated := true;
    if FStartCompletedIEN <> '' then
      LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
    if isComplete then
    begin
      if assigned(MagKatEngineProcessingCompletedEvent)
        and (AlertDoneOnComplete) then
      begin
        MagKatEngineProcessingCompletedEvent(self, FLastProcessedIen,
          'Completed processing TXT files on platter [' + platter.PlatterSerialNumber + ']');
      end;
    end
    else 
    begin
      if assigned(OnMagKatPlatterStatusUpdateUpdateEvent) then
        OnMagKatPlatterStatusUpdateUpdateEvent(self, platter);

      if assigned(MagKatEngineProcessingStoppedEvent) and
        (AlertDoneOnComplete) then
        MagKatEngineProcessingStoppedEvent(self, FLastProcessedIen);
    end;
  end;
end;

{*
 Load the TXT files from the platter file on disk
 @param Platter The platter to load from the file on disk
 @return A list of string filenames of the TXT files contained on the platter
 }
function TMagKatEngine.LoadPlatterFiles(Platter : TMagKatPlatter) : TStrings;
var
  platterFilename : String;
  txtFile : TextFile;
  TXTLine : string;
  Files : TStrings;
  totalTXTFiles : integer;
begin
  totalTXTFiles := 0;
  Files := TStringList.Create();
  result := Files;

  platterfilename := FMagKatJukeboxFileEngine.PlatterDirectory + 'Platter Detail Report ' +
    platter.PlatterName + ' (' + platter.PlatterSerialNumber + ').txt';

  LogProcessMessage('Loading file list from platter [' + platter.PlatterDisplayName + ']');
  // check to see if platter file exists
  if not FileExists(platterFilename) then
  begin
    flogger.Error('Platter file [' + platterFilename + '] does not exist, cannot process platter.');
    FErrorLogger.Error('Platter file [' + platterFilename + '] does not exist, cannot process platter.');
    result := nil;
    exit;
  end;
  try
    try


      assignFile(TxtFile, platterfilename);
      reset(TxtFile);
      while not eof(TxtFile) do
      begin
        ReadLn(TxtFile, TXTLine);
        if UpperCase( ExtractFileExt(txtline)) = '.TXT' then
        begin
          totalTXTFiles := totalTXTFiles + 1;
          files.Add(txtline);
        end;
      end
    except
      on E : exception do
      begin
        FLogger.Error('Error reading platter file [' + platterFilename + '], ' + e.Message);
        FErrorLogger.Error('Error reading platter file [' + platterFilename + '], ' + e.Message);
      end;
    end;
  finally
    CloseFile(txtFile);
    platter.TxtFilesOnPlatter := totalTXTFiles;
  end;
end;

{*
 Process TXT files sequentially starting at the specified IEN and in the
 specified direction
 @param StartIen The IEN of the TXT file to start with
 @param Direction The direction to process (forwards or backwards)
 @param FileLocation The types of files to process (magnetic, jukebox, both)
 }
procedure TMagKatEngine.ProcessSequentially(StartIen : string;
  Direction : TMagKatProcessDirection; FileLocation : TMagKatTXTFileLocation);
var
  msg : string;
  ien : integer;
  processResponse : String;
  isComplete : boolean;
begin
  FIsProcessingPlatters := false;
  if FCurrentProcessingProgress = nil then
    FCurrentProcessingProgress := TMagKatProcessingProgress.Create()
  else
    FCurrentProcessingProgress.Reset();
  try
    if assigned(MagKatEngineUpdateCurrentRateEvent) then
      MagKatEngineUpdateCurrentRateEvent(self, 'Calculating processing rate...');
    if assigned(MagKatEngineUpdateRecentErrorCountEvent) then
      MagKatEngineUpdateRecentErrorCountEvent(self, 'Calculating processing rate...');
    FImageProcessedCount := 0;
    FLastUpdatedRateTime := GetTickCount;
    FProcessingPaused := false;
    FStartCompletedIEN := '';
    FLastProcessedIen := '';
    isComplete := false;
    try
      if startien <> '' then
      begin
        ien := strtoint(startien);
        if ien < 0 then
          ien := 0;
        startIen := inttostr(ien);
      end;
    except
      on E : Exception do
      begin
        FLogger.Warn('Exception converting IEN to integer [' + e.Message + ']');
        // don't want this one to show up in error log file
//        FErrorLogger.Warn('Exception converting IEN to integer [' + e.Message + ']');
      end;
    end;
    processresponse := ProcessTXTFile(startIen, direction, FileLocation);
    while (processResponse <> '-1') do
    begin
      if FProcessingPaused then
      begin
        FLogger.Info('Pausing Sequential processing because pause button was pressed');
        exit;
      end;
      processresponse := ProcessTXTFile(processresponse, direction, FileLocation);
    end;
    FLogger.Info('Processed all images sequentially, done with processing');
    isComplete := true;
  finally
    FMagSecurity.MagCloseSecurity(msg);
    if FStartCompletedIEN <> '' then
      LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
    if isComplete then
    begin
      if assigned(MagKatEngineProcessingCompletedEvent) then
      begin
        MagKatEngineProcessingCompletedEvent(self, FLastProcessedIen, 'Completed processing images');
      end;
    end
    else
    begin
      if assigned(MagKatEngineProcessingStoppedEvent) then
        MagKatEngineProcessingStoppedEvent(self, FLastProcessedIen);
    end;
  end;
end;

{*
  Process a range of TXT file IENs at a specific location
  @param startIen The IEN of the TXT file to start wtih
  @param endIen The IEN of the TXT file to end with
  @param FileLocation The types of files to process (magnetic, jukebox, both)
  @return Doesn't seem to return anything
  }
function TMagKatEngine.ProcessRange(startIen, endIen : integer;
  FileLocation : TMagKatTXTFileLocation): boolean;
var
  i : integer;
  msg : String;
  txtFileResult : boolean;
  isComplete : boolean;
begin
  FIsProcessingPlatters := false;
  if FCurrentProcessingProgress = nil then
    FCurrentProcessingProgress := TMagKatProcessingProgress.Create()
  else
    FCurrentProcessingProgress.Reset();
  try
    if assigned(MagKatEngineUpdateCurrentRateEvent) then
      MagKatEngineUpdateCurrentRateEvent(self, 'Calculating processing rate...');
    if assigned(MagKatEngineUpdateRecentErrorCountEvent) then
      MagKatEngineUpdateRecentErrorCountEvent(self, 'Calculating processing rate...');
    FImageProcessedCount := 0;
    FLastUpdatedRateTime := getTickCount();
    FProcessingPaused := false;
    FStartCompletedIEN := '';
    FLastProcessedIen := '';
    startIen := startIen -1;
    if startIen < 0 then
      startIen := 0;
    endIen := endIen - 1;
    isComplete := false;
    for i := startIen to endIen do
    begin
      if ProcessTXTFile(inttostr(i), MagKatProcessForwards, FileLocation) = '-1' then
      begin
        FLogger.Info('Reached the end of the images in VistA, completed processing');
        exit;
      end;
      if FProcessingPaused then
      begin
        FLogger.Info('Pausing Range processing because pause button was pressed');
//      FLastProcessedIen := inttostr(i);
        exit;
      end;
    end;
    isComplete := true;
  finally
    FMagSecurity.MagCloseSecurity(msg);
    if FStartCompletedIEN <> '' then
      LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
    if isComplete then
    begin
      if assigned(MagKatEngineProcessingCompletedEvent) then
        MagKatEngineProcessingCompletedEvent(self, FLastProcessedIen, 'Completed processing range [' + inttostr(startIen) + ' to ' + inttostr(endIen) + ']');
    end
    else
    begin
      if assigned(MagKatEngineProcessingStoppedEvent) then
        MagKatEngineProcessingStoppedEvent(self, FLastProcessedIen);
    end;
  end;
end;

{*
 Process a TXT file from a platter
 @param FullFilename : The filename from the platter to process
 @return True if the file was processed (or has already been procesed), false
 if the file was not processed
 }
function TMagKatEngine.ProcessPlatterTXTFile(FullFilename : String;
  PlatterSerial : String; var errorMsg : String) : TMagKatFileProcessedAction;
var
  rpcResponse : string;
  current, filename, imgLocation : string;
  magKatImgLocation : TMagKatTXTFileLocation;
  processImgResult : TMagKatFileProcessedAction;
  imgFile, imgFileNoExtension : String;
  currentIen : integer;
  duplicateField, IQField, UpdatableField : string;
begin
  errorMsg := '';
  FImageProcessedCount := FImageProcessedCount + 1;
  if FProcessingDelay > 0 then
  begin
    FLogger.info('Sleeping for [' + inttostr(FProcessingDelay) + '] ms');
    Sleep(FProcessingDelay);
  end;
  FLogger.Info('Processing File  [' + FullFilename + '], checking if network share is known');
  if not isShareKnown(FullFilename) then
  begin
    result := MagKatProcessError;
    errorMsg := 'Platter [' + PlatterSerial + '], File [' + FullFilename +
      '] is on unknown share (no credentials to access file), skipping file';
    FLogger.Error(errorMsg);
    FErrorLogger.Error(errorMsg);
    LogProcessMessage(errorMsg);
    FCurrentProcessingProgress.FUnknownShareSkippedFiles := FCurrentProcessingProgress.FUnknownShareSkippedFiles + 1;
    if assigned(MagKatEngineCurrentIENChangedEvent) then
      MagKatEngineCurrentIENChangedEvent(self, '');
    exit;
  end;
  FLogger.Info('Checking VistA for file location details');
  imgFile := ExtractFileName(FullFilename);
  imgFileNoExtension := MagPiece(imgFile, '.', 1);
  if FBroker.findImageTextFile(imgFileNoExtension, rpcResponse) then
  begin
    FLogger.info('RPC Response [' + rpcResponse + ']');
    LogProcessMessage(rpcResponse);

    current := MagPiece(rpcResponse, '|', 1);
    filename := MagPiece(rpcResponse, '|', 2);
    imgLocation := MagPiece(rpcResponse, '|', 3);
    duplicateField := MagPiece(rpcResponse, '|', 4);
    IQField := MagPiece(rpcResponse, '|', 5);
    UpdatableField := MagPiece(rpcResponse, '|', 6);

    if Pos('-',filename) = 1 then
    begin
      if Pos('-23', filename) = 1 then // Image filename not found on VistA
      begin
        result := MagKatNA;
        FCurrentProcessingProgress.FSkippedFiles := FCurrentProcessingProgress.FSkippedFiles + 1;
      end
      else if Pos('-13', filename) = 1 then // image is deleted
      begin
        result := MagKatSkippingFile;
        FCurrentProcessingProgress.FSkippedFiles := FCurrentProcessingProgress.FSkippedFiles + 1;
      end
      else
      begin
        result := MagKatProcessError;;
        FCurrentProcessingProgress.FErrorFiles := FCurrentProcessingProgress.FErrorFiles + 1;
      end;
      errorMsg := 'Error response from VistA ['
        + rpcResponse + '] not processing file';
      FLogger.Error('Platter [' + PlatterSerial + '], ' + errorMsg);
      FErrorLogger.Error('Platter [' + PlatterSerial + '], ' + errorMsg);
      exit;
    end;

    if assigned(MagKatEngineCurrentIENChangedEvent) then
      MagKatEngineCurrentIENChangedEvent(self, current);

    magKatImgLocation := getTxtFileLocation(imgLocation);
    currentIen := strtoInt(current);

    if duplicateField = 'D' then
    begin
      errorMsg := 'Skipping file [' + current +'] because the duplicate flag is set';
      FLogger.Info(errorMsg);
      result := MagKatSkippingFile;
      if FStartCompletedIEN <> '' then
        LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
      FStartCompletedIEN := '';    
    end
    else if IQField = 'IQ' then
    begin
      errorMsg := 'Skipping file [' + current +'] because the IQ flag is set';
      FLogger.Info(errorMsg);
      result := MagKatSkippingFile;
      if FStartCompletedIEN <> '' then
        LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
      FStartCompletedIEN := '';
      FCurrentProcessingProgress.FSkippedFiles := FCurrentProcessingProgress.FSkippedFiles + 1;
    end

    else if FMagKatJukeboxFileEngine.isFileProcessed(currentIen) then
    begin
      // we are skipping this file because it is already completed
      errorMsg := 'Skipping file [' + current + '], already completed by MagKat';
      FLogger.Info(errorMsg);
      if FStartCompletedIEN <> '' then
        LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
      FStartCompletedIEN := '';
      result := MagKatPreviouslyProcessed;
      FCurrentProcessingProgress.FAlreadyProcessedFiles := FCurrentProcessingProgress.FAlreadyProcessedFiles + 1;
    end
    else if UpdatableField = '0' then
    begin
      // if we got here then the file does not need to be processed but our
      // log does not know that.
      errorMsg := 'Skipping file [' + current +'], the RPC indicates all fields have been updated';
      FLogger.Info(errorMsg);
      result := MagKatPreviouslyProcessed;
      // continuing the processed log since this file is considered done.
      if FStartCompletedIEN = '' then
          FStartCompletedIEN := current;

      FCurrentProcessingProgress.FAlreadyProcessedFiles := FCurrentProcessingProgress.FAlreadyProcessedFiles + 1;
    end
    // platter file will now contain UNC paths, compare against those
    else if UpperCase(filename) <> uppercase(FullFilename) then
    begin
      errorMsg := 'Filename from VistA ['
        + filename + '] does not match [' + FullFilename + '] from Platter file, skipping';
      FLogger.Warn('Platter [' + PlatterSerial + '], ' + errorMsg);
      if FStartCompletedIEN <> '' then
        LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
      FStartCompletedIEN := '';
      result := MagKatNA;
      FCurrentProcessingProgress.FUnknownShareSkippedFiles := FCurrentProcessingProgress.UnknownShareSkippedFiles + 1;
    end
    else if (magKatImgLocation = MagKatMagnetic) or
      (magKatImgLocation = MagKatJukebox) then
    begin
      result := readTXTFileAndUpdateVista(current, filename, errorMsg,
        PlatterSerial);
      if result = MagKatProcessedSuccessfully then
      begin
        if FStartCompletedIEN = '' then
          FStartCompletedIEN := current;
        if assigned(MagKatEngineFileProcessedEvent) then
          MagKatEngineFileProcessedEvent(self, current, magKatImgLocation);
        result := MagKatProcessedSuccessfully;
        FCurrentProcessingProgress.FProcessedFiles := FCurrentProcessingProgress.FProcessedFiles + 1;
      end
      else if result = MagKatFileIOError then
        FCurrentProcessingProgress.FIOErrors := FCurrentProcessingProgress.FIOErrors + 1
      else
      begin
        FCurrentProcessingProgress.FErrorFiles := FCurrentProcessingProgress.FErrorFiles + 1;
        //FCompletedLogger.Info();
      end;
    end
    else
    begin
      FLogger.Info('File from VistA [' + filename + '] has location [' + imgLocation + '] and cannot be processed, skipping');
      if FStartCompletedIEN <> '' then
        LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
      FStartCompletedIEN := '';
      result := MagKatProcessError;
      errorMsg := 'Image is on location [' + imgLocation + '] and cannot be processed';
      FCurrentProcessingProgress.FOfflineSkippedFiles := FCurrentProcessingProgress.FOfflineSkippedFiles + 1;
    end;
  end;
  CalculateCurrentProcessingRate();
end;

{ Returns true when reached the last image, false if more images available}
{ Returns the next IEN to process, or -1 if complete}
{*
 Processes a TXT file from the file IEN
 @param IEN The IEN of the TXT file to process
 @param Direction The direction to process (forwards or backwards)
 @param LocationToProcess The types of files to process (magnetic, jukebox, both)
 @return -1 if processing is complete, otherwise returns the IEN that was processed
 }
function TMagKatEngine.ProcessTXTFile(IEN : String; Direction : TMagKatProcessDirection;
  LocationToProcess : TMagKatTXTFileLocation) : string;
var
  rpcResponse : string;
  current, filename, imgLocation : string;
  strDirection : string;
  magKatImgLocation : TMagKatTXTFileLocation;
  processImgResult : TMagKatFileProcessedAction;
  errMsg : String;
begin
  FImageProcessedCount := FImageProcessedCount + 1;
  if Direction = MagKatProcessForwards then
    strDirection := '1'
  else
    strDirection := '-1';
  if FProcessingDelay > 0 then
  begin
    FLogger.info('Sleeping for [' + inttostr(FProcessingDelay) + '] ms');
    Sleep(FProcessingDelay);
  end;
  FLogger.Info('Processing TXT file IEN [' + IEN + '], about to scan for new TXT file');
  if FBroker.scanMagScanImageTextFiles(IEN, strDirection, rpcResponse) then
  begin
    FLogger.info('RPC Response [' + rpcResponse + ']');
    LogProcessMessage(rpcResponse);

    if rpcResponse = '-1,Done' then
    begin
      result := '-1';
      exit;
    end;

    current := MagPiece(rpcResponse, ',', 1);
    filename := MagPiece(rpcResponse, ',', 2);
    imgLocation := MagPiece(rpcResponse, ',', 3);
    result := current;

    if assigned(MagKatEngineCurrentIENChangedEvent) then
      MagKatEngineCurrentIENChangedEvent(self, current);

    magKatImgLocation := getTxtFileLocation(imgLocation);

    // check to see if the image location is equal to the specified location
    // or if the specified location is all, then check to be sure the image
    // location is magnetic or jukebox (can't process offline or invalid)
    if (MagKatImgLocation = LocationToProcess) or
      ((LocationToProcess = MagKatAll) and
        ((magKatImgLocation = MagKatMagnetic) or
         (magKatImgLocation = MagKatJukebox))) then
//    if (LocationToProcess = MagKatAll) or (magKatImgLocation = LocationToProcess) then
    begin
      processImgResult := readTXTFileAndUpdateVista(current, filename, errMsg);
      if processImgResult = MagKatProcessedSuccessfully then
      begin
        FCurrentProcessingProgress.FProcessedFiles := FCurrentProcessingProgress.FProcessedFiles + 1;
        if FStartCompletedIEN = '' then
          FStartCompletedIEN := current;
        if assigned(MagKatEngineFileProcessedEvent) then
          MagKatEngineFileProcessedEvent(self, current, magKatImgLocation);
      end
      else
      begin
        FCurrentProcessingProgress.FErrorFiles := FCurrentProcessingProgress.ErrorFiles + 1;
        FRecentFailedImageCount := FRecentFailedImageCount + 1;
        //FCompletedLogger.Info();
      end;
    end
    else
    begin
      // we are skipping this file
      FLogger.Info('Skipping file [' + current + '] because it is not on selected media type');
      if FStartCompletedIEN <> '' then
        LogCompletedImages(FStartCompletedIEN, FLastProcessedIen);
      FStartCompletedIEN := '';
      if magKatImgLocation = MagKatJukebox then
        FCurrentProcessingProgress.FJukeboxSkippedFiles := FCurrentProcessingProgress.FJukeboxSkippedFiles + 1
      else if (magKatImgLocation = magkatOffline) or
        (magKatImgLocation = MagKatArchive) or
        (magKatImgLocation = MagkatInvalid) then
        FCurrentProcessingProgress.FOfflineSkippedFiles := FCurrentProcessingProgress.FOfflineSkippedFiles + 1;
    end;
  end;
  CalculateCurrentProcessingRate();
end;

{*
 Calculates the current rate of processing. The rate is only calculated every 5
 TXT files processed (not after each file)
 }
procedure TMagKatEngine.CalculateCurrentProcessingRate();
var
  TimeDifference : int64;
  ms : int64;
  msg : String;
begin
  if (FImageProcessedCount mod 5) = 0 then
  begin
    TimeDifference := (GetTickCount - FLastUpdatedRateTime);
    if TimeDifference > 5 then
    begin
      ms := trunc(TimeDifference / 5);
      msg :=  inttostr(ms) + ' ms per TXT file';
    end
    else
    begin
      msg :=  inttostr(TimeDifference) + ' ms per 5 TXT files';
    end;
    FLastUpdatedRateTime := GetTickCount();
    if assigned(MagKatEngineUpdateCurrentRateEvent) then
      MagKatEngineUpdateCurrentRateEvent(self, Msg);
  end;
  if (FImageProcessedCount mod 50) = 0 then
  begin
    msg := inttostr(FRecentFailedImageCount) +
      ' failed TXT files in the last 50 files';
    if assigned(MagKatEngineUpdateRecentErrorCountEvent) then
      MagKatEngineUpdateRecentErrorCountEvent(self, msg);
    FRecentFailedImageCount := 0;
  end;
end;

{*
  Reads the TXT file specified for the necessary fields to find, then makes the
  RPC call to VistA to update the database
  @param IEN The IEN of the TXT file working on
  @param Filename The filename to read (full path)
  @return true if the file was processed successfully, false if there was an error
 }
function TMagKatEngine.readTXTFileAndUpdateVista(IEN : String; Filename : String;
  var errMsg : String; PlatterSerial : String = '') : TMagKatFileProcessedAction;
var
  msg : string;
  txtFile : TextFile;
  TXTLine : string;
  readDicomTags : boolean;

  seriesUid, seriesnumber, imageNumber, imageUid, studyUid, documentDate1 : String;
  documentDate2, DocumentDate3, documentTime1, documentTime2 : String;
  documentTime3, documentStamp : String;

  imageDetails : String;
  stampDate, stampTime : string;
  numdate : integer;
begin
  readDicomTags := false;
  try
    if not FMagSecurity.MagOpenSecurePath(filename, msg) then
    begin
      errMsg := 'Error opening path to file [' + filename + '], ' + msg;
      flogger.Error(errMsg);
      FErrorLogger.Error(errMsg);
      result := MagKatFileIOError;
      FLastProcessedIen := ien;    
      exit;
    end;
    if not FileExists(filename) then
    begin
      errMsg := 'File [' + filename + '] does not exist';
      flogger.Error(errMsg);
      FErrorLogger.Error(errMsg);
      FLastProcessedIen := ien;
      result := MagKatFileIOError;
      exit;
    end;
    try
      assignFile(TxtFile, filename);
      reset(TxtFile);
      while not eof(TxtFile) do
      begin
        ReadLn(TxtFile, TXTLine);

        if TxtLine = '$$BEGIN DICOM DATA' then
          readDicomTags := true
        else if TxtLine = '$$END DICOM DATA' then
          readDicomTags := false
        else
        begin
          if readDicomTags then
          begin
            if pos('0020,000E|', TXTLine) > 0 then
            begin
              seriesuid := magpiece(txtline, '|', 4);
            end
            else if pos('0020,0011|', TXTLine) > 0 then
            begin
              seriesnumber := magpiece(txtline, '|', 4);
            end
            else if pos('0020,0013|', TXTLine) > 0 then
            begin
              imageNumber := magpiece(txtline, '|', 4);
            end
            else if pos('0008,0018|', TXTLine) > 0 then
            begin
              imageUid := magpiece(txtline, '|', 4);
            end
            else if pos('0020,000D|', TXTLine) > 0 then
            begin
              studyUid := magpiece(txtline, '|', 4);
            end
            else if pos('0008,0023|', TXTLine) > 0 then
            begin
              documentdate1 := magpiece(txtline, '|', 4);
            end
            else if pos('0008,0022|', TXTLine) > 0 then
            begin
              documentDate2 := magpiece(txtline, '|', 4);
            end
            else if pos('0008,0012|', TXTLine) > 0 then
            begin
              DocumentDate3 := magpiece(txtline, '|', 4);
            end
            else if pos('0008,0033|', TXTLine) > 0 then
            begin
              documentTime1 := magpiece(txtline, '|', 4);
            end
            else if pos('0008,0032|', TXTLine) > 0 then
            begin
              documentTime2 := magpiece(txtline, '|', 4);
            end
            else if pos('0008,0013|', TXTLine) > 0 then
            begin
              documentTime3 := magpiece(txtline, '|', 4);
            end;
          end;
        end;
      end;
    finally
      closefile(txtfile);
    end;

      imageDetails := seriesUid + seriesNumber + imageNumber;
      if length(imageDetails) <= 0 then
      begin
        FLogger.Info('No relevant tags found for image [' + filename + ']');
        FLastProcessedIen := ien;
        result := MagKatProcessedSuccessfully;
        exit;
      end;

      if LowerCase(documentDate1) = '<unknown>' then
      begin
        documentDate1 := '';
        documentTime1 := '';
      end;
      if LowerCase(documentTime1) = '<unknown>' then
      begin
        documentTime1 := '';
      end;
      if LowerCase(documentDate2) = '<unknown>' then
      begin
        documentDate2 := '';
        documentTime2 := '';
      end;
      if LowerCase(documentTime2) = '<unknown>' then
      begin
        documentTime2 := '';
      end;
      if LowerCase(documentDate3) = '<unknown>' then
      begin
        documentDate3 := '';
        documentTime3 := '';
      end;
      if LowerCase(documentTime3) = '<unknown>' then
      begin
        documentTime3 := '';
      end;

      stampDate := documentDate1;
      stampTime := documentTime1;

      if length(stampDate) = 0 then
      begin
        stampDate := documentDate2;
        stamptime := documentTime2;
      end;
      if length(stampDate) = 0 then
      begin
        stampDate := documentDate3;
        stamptime := documentTime3;
      end;

      stampTime := AnsiReplaceStr(stampTime, ' ', '0');
      while length(stampTime) < 6 do
      begin
        stampTime := '0' + stampTime;
      end;

      if length(stampDate) > 0 then
      begin
        numDate := strtoint(stampDate) - 17000000;
        if numDate < 0 then
          stampDate := ''
        else
          stampDate := inttostr(numDate);
      end;
      documentStamp := '';
      if (length(stampDate) > 0) and (length(stampTime) > 0) then
      begin
        documentStamp := stampDate + '.' + stampTime;
      end;

      FLogger.Debug('Update VistA database - ' + IEN);
      FLogger.Debug('Series UID:      ' + seriesUid);
      FLogger.Debug('Series Number:   ' + seriesnumber);
      FLogger.Debug('Image Number:    ' + imageNumber);
      FLogger.Debug('Image UID:       ' + imageUid);
      FLogger.Debug('Study UID:       ' + studyUid);
      FLogger.Debug('Content d/t:     ' + documentDate1 + '-' + documenttime1);
      FLogger.Debug('Acquisition d/t: ' + documentDate2 + '-' + documenttime2);
      FLogger.Debug('Creation d/t:    ' + documentDate3 + '-' + documenttime3);
      FLogger.Debug('Document Date:   ' + documentStamp);

      if Fbroker.updateVistATxtFields(ien, seriesuid, seriesnumber, imagenumber,
        imageuid, studyuid, documentStamp, msg) then
        result := MagKatProcessedSuccessfully
      else
        result := MagKatProcessError;
      FLastProcessedIen := ien;
//      LogProcessMessage(msg);
  except
    on E : Exception do
    begin
      if PlatterSerial = '' then
        errMsg := 'Error reading TXT file and updating VistA, ' + e.Message
      else
        errMsg := 'Platter [' + PlatterSerial + '], Error reading TXT file and updating VistA, ' + e.Message;
      FLogger.Error(errMsg);
      FErrorLogger.Error(errMsg);
      result := MagKatProcessError;
      FLastProcessedIen := ien;
    end;
  end;
end;

{*
 Set the credentials necessary to access the image shares for the site
 @param Username The username to use if the username is not specified in the
 specific share
 @param Password The password to ues if the password is not specified in the
 specific share
 @param Shares The network shares from the VistA database
 }
procedure TMagKatEngine.SetImageShareCredentials(Username, Password : String; Shares : TStrings);
begin
  FUsername := username;
  FPassword := Decrypt(Password);
  FMagSecurity.UserName := FUsername;
  FMagSecurity.Password := FPassword;
  FMagSecurity.ShareList := shares;    
end;

{*
 Logs a message to the parent (if the event is assigned)
 @param Msg The message to display
 }
procedure TMagKatEngine.LogProcessMessage(Msg : string);
begin
  if assigned(OnMagKatEngineProcessMessage) then
    OnMagKatEngineProcessMessage(self, msg);
end;

{*
 Stops processing
 @return Returns nothing
 }
function TMagKatEngine.stopProcessing() : string;
begin
  FProcessingPaused := true;
end;

{*
 Clears the image share credentials from the security object
 }
procedure TMagKatEngine.ClearImageShareCredentials();
var
  msg : string;
begin
  if FMagSecurity <> nil then
  begin
    FMagSecurity.MagCloseSecurity(msg);
    FLogger.info('Closed image shares, [' + msg + ']');
    FMagSecurity.UserName := '';
    FMagSecurity.Password := '';
    if FMagSecurity.ShareList <> nil then
      FMagSecurity.ShareList.Clear();
  end;
end;

{*
 Converts the location of the TXT file in string format into a TMagKatTXTFileLocation format
 @param VistaLocation the Location of the TXT file as a response from VistA
 @return The location in TMagKatTXTFileLocation format
 }
function TMagKatEngine.getTxtFileLocation(VistaLocation : String) : TMagKatTXTFileLocation;
begin
  if VistaLocation = 'M' then
    result := MagKatMagnetic
  else if VistaLocation = 'W' then
    result := MagKatJukebox
  else if VistaLocation = 'O' then
    result := MagKatOffline
  else if VistaLocation = 'A' then
    result := MagKatArchive
  else
    result := MagKatInvalid;
end;

{*
 Logs the range of completed files to the completed log, also notifies the
 jukebox file engine of the new range of completed files
 @param StartIEN the IEN of the first image completed
 @param FinishIEN the IEN of the last image completed
 }
procedure TMagKatEngine.LogCompletedImages(StartIEN, FinishIEN : String);
var
  strt, fin, tmp : integer;
begin
  try
    strt := strtoint(startien);
    fin := strtoint(FinishIEN);
    if strt > fin then
    begin
      tmp := strt;
      strt := fin;
      fin := tmp;
    end;
    FCompletedLogger.Info(inttostr(strt) + '-' + inttostr(fin));
    FMagKatJukeboxFileEngine.addCompletedRange(strt, fin, true);
  except
    on E :Exception do
    begin
      FLogger.Error('TMagKatEngine.LogCompletedImages, ' + e.Message);
      FErrorLogger.Error('TMagKatEngine.LogCompletedImages, ' + e.Message);
    end;
  end;
end;

{*
 Converts a file processed action result to a string value
 @param Action The action to convert
 @return The string representation of the action
 }
function TMagKatEngine.MagKatFileProcessedActionToString(Action : TMagKatFileProcessedAction) : string;
begin
  result := '';
  if action = MagKatProcessedSuccessfully then
    result := 'Processed Successfully'
  else if action = MagKatPreviouslyProcessed then
    result := 'Previously Processed'
  else if action = MagKatProcessError then
    result := 'Error'
  else if action = MagKatFileIOError then
    result := 'File I/O Error'
  else if action = MagKatSkippingFile then
    result := 'Skipped Integrity File'
  else if action = MagKatNA then
    result := 'N/A';
end;

{*
 Checks to see if the share the file is on is contained in the list of
 shares the security component knows about
 @param Filename The name of the file to check if the share is available
 @return true if the share is known, false otherwise
 }
function TMagKatEngine.isShareKnown(Filename : String) : boolean;
var
  i : integer;
  shareName : string;
  sName : string;
begin
  result := false;
  if FMagSecurity = nil then
    exit;
  if FMagSecurity.ShareList = nil then
    exit;
  shareName := UpperCase( FMagSecurity.ParseServerShare(Filename));
  for i := 0 to FMagSecurity.ShareList.Count - 1 do
  begin
    sName := uppercase(MagPiece(FMagSecurity.ShareList[i], '^', 2));
    if sName = shareName then
    begin
      result := true;
      exit;
    end;
  end;
end;

{*
 Gets the details of the last processing status
 @return The status description
}
function TMagKatEngine.getCurrentProcessingProgressDetails() : string;
begin
  if FCurrentProcessingProgress = nil then
    result := 'No processing results'
  else
  begin
    if (FCurrentProcessingProgress.FPlattersProcessed > 1) and
      (FIsProcessingPlatters) then
    begin
      result := result + 'Platters Processed:' + #9 + #9  + '     ' + inttostr(FCurrentProcessingProgress.FPlattersProcessed) + #13;
    end;
    result := result + 'Processed Files: ' + #9 + #9 + '     ' + inttostr(FCurrentProcessingProgress.FProcessedFiles) + #13;
    result := result + 'Error (Failed Files): ' + #9 + #9 + '     ' + inttostr(FCurrentProcessingProgress.FErrorFiles) + #13;
    if FIsProcessingPlatters then
    begin
      result := result + 'Error (I/O File Errors):' + #9 + #9 + '     '  + inttostr(FCurrentProcessingProgress.FIOErrors) + #13;
      result := result + 'Skipped (Offline Files):' +#9 + #9  + '     ' + inttostr(FCurrentProcessingProgress.FOfflineSkippedFiles) + #13;
      result := result + 'Skipped (File Previously Processed): '  + inttostr(FCurrentProcessingProgress.FAlreadyProcessedFiles) + #13;
      result := result + 'Skipped (File on Unknown Share):' + '     ' + inttostr(FCurrentProcessingProgress.FUnknownShareSkippedFiles) + #13;
      result := result + 'Skipped (Integrity Files):' + #9 + #9 + '     ' + inttostr(FCurrentProcessingProgress.FSkippedFiles) + #13;
    end
    else
    begin
      result := result + 'Skipped (Files on Jukebox): ' + #9 + '     ' +
        inttostr(FCurrentProcessingProgress.FJukeboxSkippedFiles + FCurrentProcessingProgress.FOfflineSkippedFiles) + #13;
    end;
  end;
end;

{*
 Creates a new processing progress
 }
constructor TMagKatProcessingProgress.Create();
begin
  Reset();
end;

{*
  Resets the values of the progress report
 }
procedure TMagKatProcessingProgress.Reset();
begin
  FProcessedFiles := 0;
  FErrorFiles := 0;
  FJukeboxSkippedFiles := 0;
  FOfflineSkippedFiles := 0;
  FUnknownShareSkippedFiles := 0;
  FAlreadyProcessedFiles := 0;
  FIOErrors := 0;
  FPlattersProcessed := 0;
  FSkippedFiles := 0;
end;

end.
