{*
 Contains classes and types for jukebox file platter management
 }
unit cMagKatJukeboxFileEngine;

interface

uses classes, sysutils, dialogs, cMagKatPlatter, cMagKatUtils, TLoggerUnit;

type
  /// Contains a range of completed files
  TMagKatCompletedFiles = class
  private
    /// The start range of the completed files
    FStartRange : integer;
    /// The end range of the completed files
    FEndRange : integer;
  public
    constructor Create(StartRange, EndRange : integer);
    property StartRange : integer read FStartRange write FStartRange;
    property EndRange : integer read FEndRange write FEndRange;
  end;

type
  /// Engine to handle reading and writing files for managing platters
  TMagKatJukeboxFileEngine = class
  private
    /// List of range of completed files
    FCompletedRange : TList;
    /// General information logger
    FLogger : TLogger;
    /// Error logger
    FErrorLogger : TLogger;
    /// Keeps track if the list is sorted
    FIsSorted : boolean;
    /// The directory where the platters are stored
    FPlatterDirectory : string;
    /// If a platter has been updated and needs to be written to disk
    FPlattersUpdated : boolean;
    /// Hold the platter update file used to store the updates to a platter file
    FPlatterUpdateFile : string;
    function createPlatter(PlatterLine : String) : TMagKatPlatter;
    function readPlatterFile(Filename : String; MergePlatters : TList) : TList;
    function findPlatter(PlatterSerial : String; PlatterList : TList) : TMagKatPlatter;
    procedure mergePlatter(OldPlatter, NewPlatter : TMagKatPlatter);


  public
    constructor Create(logger : TLogger; ErrorLogger : TLogger;
       PlatterDirectory : string);
    destructor Destroy;  override;
    function readPlatterFromFile(PlatterFile : String;
      UpdatePlatterFile : String) : TList;
    procedure readCompletedFile(CompletedFilename : String);
    procedure addCompletedRange(range : TMagKatCompletedFiles;
      unique : boolean = true); overload;
    procedure addCompletedRange(StartRange, EndRange : integer;
      unique : boolean = true); overload;
    function isFileProcessed(IEN : integer) : boolean;
    procedure writeProcessedToFile(Filename : String);
    procedure writePlatterstoFile(UpdatePlatterFile : String; platters : TList);
    function writePlatterReport(Platter : TMagKatPlatter; FileList : TStrings) : boolean;
    function getPlatterReport(Platter : TMagKatPlatter) : TStrings;
    procedure clearCompletedFile();
    procedure writePlattertoFile(Platter : TMagKatPlatter);

    function getCompletedFileCount() : integer;
    function getCompletedRangeCount() : integer;

    /// The directory where the platters are stored
    property PlatterDirectory : String read FPlatterDirectory;
    /// If a platter has been updated and needs to be written to disk
    property PlattersUpdated : boolean read FPlattersUpdated write FPlattersUpdated;

    /// This is a property available ONLY for debug purposes and should not
    /// be used for production use!
    property CompletedRange : TList read FCompletedRange;

  end;

{*
  Compares items in the list to sort by completed file
  @param Item1 first item to compare
  @param Item2 second item to compare
  @return 1 for higher, -1 for lower, 0 for equal
 }
function compareCompletedFiles(Item1, Item2: Pointer) : integer;

implementation

{*
  Create a new TMagKatCompletedFiles
  @param StartRange start of range of completed files
  @param EndRange end of range of completed files
 }
constructor TMagKatCompletedFiles.Create(StartRange, EndRange : integer);
begin
  inherited Create;
  FStartRange := StartRange;
  FEndRange := EndRange;
end;

{*
  Create a new TMagKatJukeboxFileEngine
  @param logger information logger
  @param ErrorLogger error logger
  @param PlatterDirectory directory where the platter files are stored
 }
constructor TMagKatJukeboxFileEngine.Create(logger : TLogger;
  ErrorLogger : TLogger; PlatterDirectory : string);
begin
  inherited Create;
  FLogger := logger;
  FIsSorted := false;
  FErrorLogger := ErrorLogger;
  FPlatterDirectory := PlatterDirectory;
  FCompletedRange := TList.Create();
end;

{*
 Destructor to destroy instantiated variables
 }
destructor TMagKatJukeboxFileEngine.Destroy;
begin
  inherited destroy;
  if FCompletedRange <> nil then
    FreeAndNil(FCompletedRange);
end;

{*
  Checks to see if a file IEN has already been processed
  @param IEN the IEN of the TXT file to check for
  @return True if the file has been processed, false if it has not been processed
 }
function TMagKatJukeboxFileEngine.isFileProcessed(IEN : integer) : boolean;
var
  i : integer;
  completedFiles : TMagKatCompletedFiles;
  rangeStart, rangeEnd, rangeSize : integer;
  searchLoc : integer;
begin
  // if things have been added since the last sort, sort it now
  if not FIsSorted then
    FCompletedRange.Sort(@compareCompletedFiles);
  FIsSorted := true;

  // brute force - crappy (although because of optimization of creating list,
  // this has not been too bad)

  for i := 0 to FCompletedRange.Count - 1 do
  begin
    completedFiles := FCompletedRange.Items[i];
    if (completedFiles.StartRange <= IEN) and
      (completedFiles.EndRange >= IEN) then
    begin
      result := true;
      exit;
    end;
  end;
  result := false;
end;

{*
  Add a range of completed files
  @param range The range to add
  @param unique Determines if this should be added as a unique instance or not
 }
procedure TMagKatJukeboxFileEngine.addCompletedRange(range : TMagKatCompletedFiles;
  unique : boolean = true);
var
  i : integer;
  completedFiles : TMagKatCompletedFiles;
begin
  if unique then
  begin
    if not FIsSorted then
      FCompletedRange.Sort(@compareCompletedFiles);
    FIsSorted := true;

    //for i := 0 to FCompletedRange.Count - 1 do
    for i := FCompletedRange.Count - 1 downto 0 do
    begin
      completedFiles := FCompletedRange.Items[i];
      // if the new item is before this item
      if range.EndRange <= completedFiles.StartRange then
      begin
        if range.EndRange = completedFiles.StartRange then
        begin
          // JMW 7/31/08 - only update the start range if the new one is
          // before the current start range
          if range.StartRange < completedFiles.StartRange then
            completedFiles.StartRange := range.StartRange;
          FreeAndNil(range);
          exit;
        end;
        FCompletedRange.Insert(i + 1, range);
        exit;
      end;
      if (range.StartRange <= (completedFiles.EndRange + 1)) and
        (range.StartRange >= (completedFiles.StartRange - 1)) then
      begin
        if completedFiles.EndRange < range.EndRange then
          completedFiles.EndRange := range.EndRange;
        if completedFiles.StartRange > range.StartRange then
          completedFiles.StartRange := range.StartRange;
        FreeAndNil(range);
        exit;
      end;
      if range.FEndRange <= completedFiles.FEndRange then
      begin
        if range.FStartRange >= completedFiles.FStartRange then
        begin
          FreeAndNil(range);
          // do nothing, already there
        end
        // the new end range is equal or higher than current start
        // so set the current start to the new start (in case new start is before current start)

        // JMW 7/31/08 - need to be sure the new end range is inside
        // the current range (new end > current start)
        else if range.FEndRange >= completedFiles.FStartRange then
        begin
          // JMW 7/31/08 - only set the new start if less than current start
          if range.StartRange < completedFiles.StartRange then
            completedFiles.StartRange := range.StartRange;
          FreeAndNil(range);
        end
        // the ranges do not overlap, insert new one
        else
        begin
          FCompletedRange.Insert(i + 1, Range);
        end;
        exit; // range is added or already exists
      end;
      if (range.StartRange <= (completedFiles.StartRange)) and
        (range.EndRange >= completedFiles.EndRange) then
      begin
        if range.StartRange < completedFiles.StartRange then
          completedFiles.StartRange := range.StartRange;
        if range.EndRange > completedFiles.EndRange then
          completedFiles.EndRange := range.EndRange;
        FreeAndNil(range);
        exit;
      end;
    end; {end of for loop}
    // add at end if not found in middle
    //FCompletedRange.Add(Range);
    FCompletedRange.Insert(0, Range);
  end
  else
  begin
    FCompletedRange.Add(range);
    FIsSorted := false;
  end;
end;

{*
  Add a completed range to the list of completed files
  @param StartRange the begining of the completed range
  @param EndRange The end of the completed range
  @param unique Determines if this should be added as a unique instance or not
 }
procedure TMagKatJukeboxFileEngine.addCompletedRange(StartRange, EndRange : integer;
  unique : boolean = true);
var
  completedFile : TMagKatCompletedFiles;
begin
  completedFile := TMagKatCompletedFiles.Create(StartRange, EndRange);
  addCompletedRange(completedFile, unique);
end;

{*
  Read through a file in the completed file format
  @param CompletedFilename The name of the file to read
 }
procedure TMagKatJukeboxFileEngine.readCompletedFile(CompletedFilename : String);
var
  txtFile : TextFile;
  TXTLine : string;
  numRange : String;
  completedFile : TMagKatCompletedFiles;
  startRange, endRange : integer;
begin
  // cleanup completed file range if user logged in previously.
  // this is ok since the file is written to immediately when an entry occurs
  clearCompletedFile();
  FCompletedRange := TList.Create();
  FLogger.Info('Reading completed files from [' + CompletedFilename + ']');
  if not FileExists(CompletedFilename) then
  begin
     FLogger.Info('Completed file does not exist');
     exit;
  end;
  try
    try
      assignFile(TxtFile, CompletedFilename);
      reset(TxtFile);
      while not eof(TxtFile) do
      begin
        ReadLn(TxtFile, TXTLine);
        numRange := MagPiece(TXTLine, ']', 2);
        numRange := trim(numRange);
        startRange := strtoint(MagPiece(numRange, '-', 1));
        endRange := strtoint(MagPiece(numRange, '-', 2));
        if startRange = 5281865 then
        begin
          startRange := 5281865;
        end;
        addCompletedRange(startRange, endRange, true);
      end;
      FLogger.Info('Found [' + inttostr(FCompletedRange.Count) + '] ranges of completed files');
      FCompletedRange.Sort(@compareCompletedFiles);
      FIsSorted := true;
    except
      on E : Exception do
      begin
        FLogger.Error('Error reading completed list of files [' + e.Message + ']');
        FErrorLogger.Error('Error reading completed list of files [' + e.Message + ']');
      end;
    end;
  finally
    CloseFile(txtFile);
  end;    
end;

{*
 Read a list of platters and an update list of platters
 @param PlatterFile The original platter file to read from
 @param UpdatePlatterFile A file containing updates to the platter list (can be
 empty if no update file list exists)
 @return List of platters
 }
function TMagKatJukeboxFileEngine.readPlatterFromFile(PlatterFile : String;
  UpdatePlatterFile : String) : TList;
var
  PlatterList : TList;
begin
  FPlatterUpdateFile := UpdatePlatterFile;
  PlatterList := readPlatterFile(platterfile, nil);
  result := readPlatterFile(UpdatePlatterFile, PlatterList);
end;

{*
 Reads a platter file
 @param Filename The file to read the platters from
 @param MergePlatters Platters to merge with the new updates (can be nil)
 @return List of platters
 }
function TMagKatJukeboxFileEngine.readPlatterFile(Filename : String;
  MergePlatters : TList) : TList;
var
  txtFile : TextFile;
  TXTLine : string;
  firstLine : boolean;
  oldPlatter, platter : TMagKatPlatter;
begin

  if MergePlatters = nil then
    result := tList.Create()
  else
    result := MergePlatters;
  if not FileExists(filename) then
    exit;

  firstLine := true;
  try
    try
      assignFile(TxtFile, Filename);
      reset(TxtFile);
      while not eof(TxtFile) do
      begin
        oldPlatter := nil;
        ReadLn(TxtFile, TXTLine);
        if not firstline then
        begin
          platter := createPlatter(TXTLine);
          // find the old platter that matches the new platter
          oldPlatter := findPlatter(platter.PlatterSerialNumber, result);
          // if no old platter found (shouldn't happen), add the new platter
          if oldPlatter = nil then
          begin
            result.Add(platter);
          end
          else
          begin
            // update the existing platter on the list with the
            // new details
            mergePlatter(oldPlatter, platter);
            // JMW 9/10/08 If the old platter existed, then we are just
            // updating the old one, so get rid of the new platter
            FreeAndNil(platter);
          end;
        end;
        firstLine := false;
      end;
    except
      on E : Exception do
      begin
        FLogger.Error('Error reading platter file, "' + Filename +
          '", [' + e.Message + ']');
        FErrorLogger.Error('Error reading platter file, "' + Filename +
          '", [' + e.Message + ']');
      end;
    end;
  finally
    closeFile(txtFile);
  end;
end;

{*
 Finds a platter in the given list based on the serial number
 @param PlatterSerial the serial number of the platter to find
 @param PlatterList The list of platters
 @return The found platter or nil if no platter matches the serial number
 }
function TMagKatJukeboxFileEngine.findPlatter(PlatterSerial : String;
  PlatterList : TList) : TMagKatPlatter;
var
  i : integer;
  platter : TMagKatPlatter;
begin
  result := nil;
  for i := 0 to PlatterList.Count - 1 do
  begin
    platter := PlatterList.Items[i];
    if platter.PlatterSerialNumber = PlatterSerial then
    begin
      result := platter;
      exit;
    end;
  end;
  result := nil;
end;

{*
 Merges an existing platter with updates from a new platter
 @param OldPlatter The old existing platter
 @param NewPlatter The new platter to overwrite with
 }
procedure TMagKatJukeboxFileEngine.mergePlatter(OldPlatter, NewPlatter : TMagKatPlatter);
begin
  OldPlatter.PlatterStatus := NewPlatter.PlatterStatus;
  OldPlatter.TxtFilesOnPlatter := NewPlatter.TxtFilesOnPlatter;
  OldPlatter.TxtFilesProcessed := NewPlatter.TxtFilesProcessed;
  OldPlatter.FailedTxtFiles := NewPlatter.FailedTxtFiles;
end;

{*
 Creates a new platter based on the platter string from the PlatterWithFiles.txt
 format
 @param PlatterLine the line from a file representing a platter
 @return A Platter object
 }
function TMagKatJukeboxFileEngine.createPlatter(PlatterLine : String) : TMagKatPlatter;
var
  platter : TMagKatPlatter;
  platterName, platterSerial : String;
  platterStatusString : String;
  strTotalTXTFiles : String;
  strProcessedTXTFiles : String;
  strFailedTxtFiles : String;
begin
  PlatterName := cMagKatUtils.MagPiece(PlatterLine, #9, 1);
  PlatterSerial := cMagKatUtils.MagPiece(PlatterLine, #9, 2);
  PlatterStatusString := cMagKatUtils.MagPiece(PlatterLine, #9, 3);
  strTotalTXTFiles := cMagKatUtils.MagPiece(PlatterLine, #9, 4);
  strProcessedTXTFiles := cMagKatUtils.MagPiece(PlatterLine, #9, 5);
  strFailedTxtFiles := cMagKatUtils.MagPiece(PlatterLine, #9, 6);
  platter := TMagKatPlatter.Create(PlatterName, platterSerial, PlatterStatusString);
  if strTotalTXTFiles <> '' then
    platter.TxtFilesOnPlatter := strtoint(strTotalTXTFiles);
  if strProcessedTXTFiles <> '' then
    platter.TxtFilesProcessed := strtoint(strProcessedTXTFiles);
  if strFailedTxtFiles <> '' then
    platter.FailedTxtFiles := strtoint(strFailedTxtFiles);
  result := platter;
end;

{*
  Compare completed file entries so the item with the lowest start range is first
  @param Item1 The first item to compare
  @param Item2 The second item to compare
  @return 1 if item1 is higher, -1 if item2 is higher, 0 if they are equal
 }
function compareCompletedFiles(Item1,
  Item2: Pointer) : integer;
var
  file1, file2 : TMagKatCompletedFiles;
  startFile1, startFile2 : integer;
begin
  try
    file1 := item1;
    file2 := item2;
    startFile1 := file1.FStartRange;
    startFile2 := file2.FStartRange;
    if startFile1 > startFile2 then
      result := -1//1
    else if startFile1 < startFile2 then
      result := 1//-1
    else
      result := 0;
  except
    on E : exception do
    begin
//      FLogger.Error('Error comparing files, ' + e.Message);
    end;
  end;
end;

{*
  Writes the processed files list to a file (for testing only)
  @param Filename The filename to write the data to
 }
procedure TMagKatJukeboxFileEngine.writeProcessedToFile(Filename : String);
var
  output : TStrings;
  i : integer;
  completedFile : TMagKatCompletedFiles;
begin
  // the filename will be empty if the user didn't log into VistA (if they
  // only started the app but didn't really use it).
  if Filename = '' then
    exit;

  if Fileexists(Filename) then
  begin
    renameFile(filename, filename + '.backup');
  end;

  output := TStringList.Create();
  for i := 0 to FCompletedRange.Count -1 do
  begin
    completedFile := FCompletedRange.Items[i];
    output.Add('[ ] ' +  inttostr(completedFile.FStartRange) + '-' + inttostr(completedFile.FEndRange));
  end;
  output.SaveToFile(filename);
end;

{*
  Write the updated platter details to a file. This file appends to an existing
  file or creates a new one if necessary.
  @param platter The platter to write to the appended status file
 }
procedure TMagKatJukeboxFileEngine.writePlattertoFile(Platter : TMagKatPlatter);
var
  txtFile : TextFile;
  TXTLine : string;
  i : integer;
  prevExists : boolean;
begin
  try
    try                    
      prevExists := false;
      FLogger.Info('Writing platter [' + platter.PlatterDisplayName +
        '] to [' + FPlatterUpdateFile + ']');
      if FileExists(FPlatterUpdateFile) then
        prevExists := true;
      assignFile(TxtFile, FPlatterUpdateFile);
      if prevExists then
        Append(TxtFile)
      else
      begin
        rewrite(TxtFile);
        writeln(txtFile, 'Platter Name' + #9 + 'Platter Serial Number' + #9 +
          'Platter Status' + #9 + 'Total TXT Files' + #9 + 'Processed Files' +
          #9 + 'Failed TXT Files');
      end;
      txtLine := platter.PlatterName + #9 + platter.PlatterSerialNumber +
        #9 + platter.PlatterStatusString + #9 +
        inttostr(platter.TxtFilesOnPlatter) + #9 +
        inttostr(platter.TxtFilesProcessed) + #9 +
        inttostr(platter.FailedTxtFiles);
      writeln(txtFile, txtLine);
    except
      on E : Exception do
      begin
        FErrorLogger.Error('Error writing platter file to disk [' + e.Message + ']');
        FLogger.Error('Error writing platter file to disk [' + e.Message + ']');
      end;
    end;
  finally
    try
      CloseFile(txtFile);
    except
      on E : Exception do
      begin
        FErrorLogger.Error('Error closing platter file [' + e.Message + ']');
        FLogger.Error('Error closing platter file [' + e.Message + ']');
      end;
    end;
  end;
end;

{*
  Write the updated platter details to a file. This version overwrites any
  previous contents of the update file and rewrites all platters
  @param UpdatePlatterFile The filename to write the platter details to
 }
procedure TMagKatJukeboxFileEngine.writePlatterstoFile(UpdatePlatterFile : String;
  platters : TList);
var
  txtFile : TextFile;
  TXTLine : string;
  platter : TMagKatPlatter;
  i : integer;
begin
  try
    try
      FLogger.Info('Writing platter info to [' + UpdatePlatterFile + ']');
      assignFile(TxtFile, UpdatePlatterFile);
      rewrite(TxtFile);
      writeln(txtFile, 'Platter Name' + #9 + 'Platter Serial Number' + #9 +
        'Platter Status' + #9 + 'Total TXT Files' + #9 + 'Processed Files' +
        #9 + 'Failed TXT Files');

      FLogger.Info('Writing [' + inttostr(platters.Count) + '] platters to file');
      for i := 0 to platters.Count - 1 do
      begin
        platter := platters.Items[i];
        txtLine := platter.PlatterName + #9 + platter.PlatterSerialNumber +
          #9 + platter.PlatterStatusString + #9 +
          inttostr(platter.TxtFilesOnPlatter) + #9 +
          inttostr(platter.TxtFilesProcessed) + #9 +
          inttostr(platter.FailedTxtFiles);
        writeln(txtFile, txtLine);
      end;
    except
      on E : Exception do
      begin
        FErrorLogger.Error('Error writing platter file to disk [' + e.Message + ']');
        FLogger.Error('Error writing platter file to disk [' + e.Message + ']');
      end;
    end;
  finally
    try
      CloseFile(txtFile);
    except
      on E : Exception do
      begin
        FErrorLogger.Error('Error closing platter file [' + e.Message + ']');
        FLogger.Error('Error closing platter file [' + e.Message + ']');
      end;
    end;
  end;
end;

{*
  Writes a platter report to the jukebox drive
  @param Platter The platter for which the report is about
  @parm FileList The details about the platter
  @return True if the report was written, false if there was a problem
 }
function TMagKatJukeboxFileEngine.writePlatterReport(Platter : TMagKatPlatter;
  FileList : TStrings) : boolean;
var
  filename : String;
begin
  try
    result := false;
    if FileList = nil then
      exit;
    filename := FPlatterDirectory + platter.PlatterName + '_' +
      platter.PlatterSerialNumber + '_Report.txt';
    FLogger.info('Saving platter report to file [' + filename + ']');
    FileList.SaveToFile(filename);
    result := true;
  except
    on E : Exception do
    begin
      FLogger.Error('Error saving platter report to file [' + filename +
        '], Error=[' + e.Message + ']');
      FErrorLogger.Error('Error saving platter report to file [' + filename +
        '], Error=[' + e.Message + ']');
    end;
  end;
end;

{*
 Returns a list of strings representing a platter report
 @param Platter The platter for which to find the report of
 @return a list of strings containing the platter report
 }
function TMagKatJukeboxFileEngine.getPlatterReport(Platter : TMagKatPlatter) : TStrings;
var
  reportList : TStrings;
  filename : String;
begin
  try
    filename := FPlatterDirectory + platter.PlatterName + '_' +
      platter.PlatterSerialNumber + '_Report.txt';
    if FileExists(filename) then
    begin
      FLogger.info('Loading platter report from file [' + filename + ']');
      reportList := TStringList.Create();
      reportList.LoadFromFile(filename) ;
      result := reportList;
    end
    else
    begin
      FLogger.info('Platter report file [' + filename + '] does not exist');
      result := nil;
    end;
  except
    on E : Exception do
    begin
      FLogger.Error('Error loading platter report from file [' + filename +
        '], Error=[' + e.Message + ']');
      FErrorLogger.Error('Error loading platter report from file [' + filename +
        '], Error=[' + e.Message + ']');
    end;
  end;
end;

{*
 Clears the list of completed files.
 }
procedure TMagKatJukeboxFileEngine.clearCompletedFile();
var
  completedFiles : TMagKatCompletedFiles;
  i : integer;
begin
  if FCompletedRange <> nil then
  begin
    for i := 0 to FCompletedRange.Count - 1 do
    begin
      completedFiles := FCompletedRange[i];
      FreeAndNil(completedFiles);
    end;
    FreeAndNil(FCompletedRange);
  end;
end;

{*
  Counts the number of completed files in the list
  @return The number of files processed and in the list.
 }
function TMagKatJukeboxFileEngine.getCompletedFileCount() : integer;
var
  i : integer;
  completedFiles : TMagKatCompletedFiles;
begin
  result := 0;
  if FCompletedRange <> nil then
  begin
    for i := 0 to FCompletedRange.Count - 1 do
    begin
      completedFiles := FCompletedRange[i];
      result := result + (completedFiles.FEndRange - completedFiles.FStartRange + 1);
    end;
  end;
end;

{*
  Counts the number of ranges in the completed file range list
  @return The number of entries in the range.
 }
function TMagKatJukeboxFileEngine.getCompletedRangeCount() : integer;
begin
  result := 0;
  if FCompletedRange <> nil then
  begin
    result := FCompletedRange.Count;
  end;
end;

end.
