{*
 Contains classes and types for displaying a platter report
 }
unit fMagKatPlatterReport;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, cMagKatPlatter, StdCtrls, ComCtrls, cMagKatUtils, Menus, clipbrd;

type

  {*
    Form to display platter reports
  }
  TfrmPlatterReport = class(TForm)
    /// list to hold report contents
    lstReport: TListView;
    /// close button
    btnClose: TButton;
    /// label to show the platter name label
    lblPlatterNameLabel: TLabel;
    /// label to show the platter serial label
    lblPlatterSerialLabel: TLabel;
    /// label to show the platter name
    lblPlatterName: TLabel;
    /// label to show the platter serial number
    lblPlatterSerialNumber: TLabel;
    /// menu for platter options
    mnuPlatterOptions: TPopupMenu;
    /// menu to copy items to the clipboard
    CopyToClipboard1: TMenuItem;
    /// range label label
    lblRangeLabel: TLabel;
    /// range label
    lblRange: TLabel;
    /// button to move to the previous range
    btnPreviousRange: TButton;
    /// button to move to the next range
    btnNextRange: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure lstReportColumnClick(Sender: TObject; Column: TListColumn);
    procedure lstReportCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure FormCreate(Sender: TObject);
    procedure CopyToClipboard1Click(Sender: TObject);
    procedure lstReportKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnPreviousRangeClick(Sender: TObject);
    procedure btnNextRangeClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    /// the current platter
    FPlatter : TMagKatPlatter;
    /// Holds the current sort order
    FSortAscending : boolean;
    /// Holds the current sort column
    FSortColumn : integer;
    /// The list of files on the platter
    FFileList : TStrings;
    /// The currently shown start range of platter files
    FCurrentStartRange : integer;
    /// The currently shown end range of platter files
    FCurrentEndRange : integer;

    procedure AddFileLine(Line : String);
    procedure DisplayPlatterFiles(startRange, endRange : integer;
      showAll :boolean);
  public
    procedure ShowPlatterReport(Platter : TMagKatPlatter; FileList : TStrings);
  end;

var
  /// Platter report form instance
  frmPlatterReport: TfrmPlatterReport;

const
  /// maximum number of txt files to show in the view at one time
  RANGE_SIZE = 5000;

implementation

{$R *.dfm}

{*
  displays the report for a platter
  @param Platter The platter being displayed
  @param FileList The list of files on the platter with their status
}
procedure TfrmPlatterReport.ShowPlatterReport(Platter : TMagKatPlatter;
  FileList : TStrings);

begin
  if FileList = nil then
    exit;
  if FFileList <> nil then
    FreeAndNil(FFileList);
  FFileList := FileList;
  self.Caption := 'Platter Report - ' + platter.PlatterDisplayName;
  lblPlatterName.Caption := platter.PlatterName;
  lblPlatterSerialNumber.Caption := platter.PlatterSerialNumber;
  lstReport.Items.Clear();
  if FFileList.Count > RANGE_SIZE then
  begin
    DisplayPlatterFiles(0, RANGE_SIZE, false);
  end
  else
  begin
    DisplayPlatterFiles(0, 0, true);
  end;
  self.Show();   
end;

{*
  Display a range of files from a platter list
  @param startRange The start range of platter files to show
  @param endRange The end range of platter files to show
  @param showAll Determines if all the files on the platter should be shown
}
procedure TfrmPlatterReport.DisplayPlatterFiles(startRange, endRange : integer;
  showAll :boolean);
var
  i : integer;
begin
  FCurrentStartRange := startRange;
  FCurrentEndRange := endRange;
  Screen.Cursor := crHourGlass;
  lstReport.Items.Clear();

  lblRangeLabel.Visible := not showAll;
  lblRange.Visible := not showAll;
  btnPreviousRange.Visible := not showAll;
  btnNextRange.Visible := not showAll;
  if showAll then
  begin
    for i := 0 to FFileList.Count - 1 do
    begin
      AddFileLine(fFileList[i]);
    end;
  end
  else
  begin
    for i := FCurrentStartRange to FCurrentEndRange - 1 do
    begin
      AddFileLine(FFileList[i]);
    end;
    lblRange.Caption := 'Displaying ' + inttostr(FCurrentStartRange) +
      ' to ' + inttostr(FCurrentEndRange) + ' of ' + inttostr(FFileList.Count);
    if FCurrentStartRange <= 0 then
      btnPreviousRange.Enabled := false
    else
      btnPreviousRange.Enabled := true;
    if FCurrentEndRange >= FFileList.Count then
      btnNextRange.Enabled := false
    else
      btnNextRange.Enabled := true;
  end;
  Screen.Cursor := crDefault;
end;

{*
 Add a line to the report for a file on the platter
 @param Line The line to parse and add to the report display
 }
procedure TfrmPlatterReport.AddFileLine(Line : String);
var
  listItem : TListItem;
begin
  listItem := lstReport.Items.Add();
  listItem.Caption := ExtractFileName( MagPiece(line, #9, 1));
  listitem.SubItems.Add(MagPiece(Line, #9, 2));
  listitem.SubItems.Add(MagPiece(Line, #9, 3));

end;

{*
 Button close event
 @param Sender The object which sent the event
}
procedure TfrmPlatterReport.btnCloseClick(Sender: TObject);
begin

  self.Close;
end;

{*
 Event when a column is clicked
 @param Sender The object which sent the event
 @param Column the column which was clicked
}
procedure TfrmPlatterReport.lstReportColumnClick(Sender: TObject;
  Column: TListColumn);
var
  NewSortColumn: integer;
begin
  newSortColumn := Column.Index - 1;
  if (NewSortColumn <> FSortColumn) then
    FSortAscending := true
  else
    FSortAscending := not FSortAscending;
  FSortColumn := NewSortColumn;
  lstReport.AlphaSort();
end;

{*
 Compare the items in the list
 @param Sender The object which sent the event
 @param Item1 The first item to compare
 @param Item2 The second item to compare
 @param Data
 @param Compare the result of the comparison
 }
procedure TfrmPlatterReport.lstReportCompare(Sender: TObject; Item1,
  Item2: TListItem; Data: Integer; var Compare: Integer);
begin
  case FSortColumn of
    -1:
      compare := CompareText(Item1.Caption, Item2.Caption);
    else
      Compare := CompareText(Item1.SubItems.Strings[FSortColumn],
        Item2.SubItems.Strings[FSortColumn]);
  end;

  if (not FSortAscending) then
    Compare := Compare * -1;
end;

{*
 Occurs when the form is created
 @param Sender The object which sent the event
 }
procedure TfrmPlatterReport.FormCreate(Sender: TObject);
begin
  FSortAscending := false;
  FFileList := nil;
end;

{*
 Copy the contents of the selected file to the clipboard
 @param Sender The object which sent the event
 }
procedure TfrmPlatterReport.CopyToClipboard1Click(Sender: TObject);
var
  listItem : TListItem;
  itemText : string;
begin
  listItem := lstReport.Selected;
  if listItem = nil then
    exit;
  itemText := listitem.Caption + #9 + listitem.SubItems[0] + #9  +
    listitem.SubItems[1];
  Clipboard.SetTextBuf(pchar(itemText));  
end;

{*
  Event occurs when the user presses a key on the keyboard while viewing the
  report. If the user selects ctrl+c the selected line is copied to the
  clipboard
  @param Sender The object which sent the event
  @param Key The button selected
  @param Shift The state of the shift key
 }
procedure TfrmPlatterReport.lstReportKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssCtrl in shift) then
  begin
    if key = 67 then // Control+c
    begin
      CopyToClipboard1Click(self);
    end;
  end;

end;

{*
  Event occurs when the user presses the button to move to the previous range
  of text files.
  @param Sender The object which sent the event
 }
procedure TfrmPlatterReport.btnPreviousRangeClick(Sender: TObject);
var
  st, ed : integer;
begin
  if FCurrentStartRange > 0 then
  begin
    st := FCurrentStartRange - RANGE_SIZE;
    if st < 0 then
      st := 0;
    ed := FCurrentEndRange - RANGE_SIZE;
    DisplayPlatterFiles(st, ed, false);

  end;
end;

{*
  Event occurs when the user presses the button to move to the next range
  of text files.
  @param Sender The object which sent the event
 }
procedure TfrmPlatterReport.btnNextRangeClick(Sender: TObject);
var
  st, ed : integer;
begin
  if FCurrentEndRange < FFileList.Count then
  begin
    st := FCurrentStartRange + RANGE_SIZE;

    ed := FCurrentEndRange + RANGE_SIZE;
    if ed > FFileList.Count then
      ed := FFileList.Count;
    DisplayPlatterFiles(st, ed, false);

  end;

end;

{*
  Event occurs when the platter report form is destroyed.
  @param Sender The object which sent the event
 }
procedure TfrmPlatterReport.FormDestroy(Sender: TObject);
begin
  if FFileList <> nil then
    FreeAndNil(FFileList);
end;

end.
