{*
 Contains classes and types for making RPC calls to the VistA database
 }
Unit cMagUtilityBroker;
{*
Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:   Oct 2008           
 Site Name: Silver Spring, OIFO
 Developers: Jemmy Lin
 Verson: MAG*3.0*98
        ;; +---------------------------------------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +---------------------------------------------------------------------------------------------------+
  }

Interface

Uses
  Trpcb,
  SysUtils,
  Dialogs,
  RPCconf1,
  Controls,
  TLoggerUnit,
  Classes,
  cMagUtilityUtils;

Type

  /// Defines the possible results from a login
  TMagUtilityBrokerLoginResult = (MagUtilityLoginOK, MagUtilityLoginError, MagUtilityLoginContextFailed);
  TMagUtilityBroker = Class
  Private
    /// The broker instance to use to communicate with the VistA database
    FBroker: TRPCBroker;
    /// general information logger
    FLogger: TLogger;
    /// Error logget
    FErrorLogger: TLogger;

    Function getConnected(): Boolean;
    Function GetUserFullName(): String;
  Public
    Constructor Create(Logger: TLogger; ErrorLogger: TLogger);
    Destructor Destroy; Override;

    Function LogIn(Var Server, Port: String; Var Errmsg: String): TMagUtilityBrokerLoginResult;
    Function Logout(): Boolean;
    Function getSiteParameterCredentials(Var Username, Password, PrimarySiteStationNumber: String): Boolean;
    Function getUserInfo(Var Sitecode, uname, Place: String): Boolean;
    Function GetNetworkLocations(Var Shares: TStrings): Boolean;
    Function getRAIDLocations(Var Shares: TStrings): Boolean;

    Procedure RPMaggInstall(Var Fstat: Boolean; Var Flist: Tstringlist);
    Procedure RPMagVersionStatus(Var Status: String; Version: String);

    Function getUserKeys(Var Keys: Tstringlist): Boolean;
    Function getRequiredContext(): String;
    Function FnmGetIEN(Var chkvar: String): Integer;
    Function FnmGetNode(Var chkvar: String): String;
    Function MagFindfile(Magien, Filetype: String): String;
    Function MagInstall(Var Version: String): Boolean;
    Function setNETloc(Magien: String; LocIEN: String): Boolean;
    Function queueJB(Magien: String): Boolean;
    Function unsetNETloc(Filename, filextn: String; Magien: Integer): Boolean;
    Function DTRangeIEN(Var DTrange, limit: String): String;
    Function NtLocDel(Var Ien, Place, stp: String): String;
    Function LstOflJB(Var JBlst: TStrings): Boolean;
    Function BKOnlJB(Var OflJB: String): Boolean;
    Function JBOffLn(Var TxtLn, Platter: String): Boolean;
    Function FETCH(Var MAGR: TStrings; Magien, LocIEN: String): Boolean;

    /// True if the broker is connected, false otherwise
    Property Connected: Boolean Read getConnected;
    /// The connected users full name
    Property UserFullName: String Read GetUserFullName;

  End;

Const

  /// Required context for running MagUtility
  MAG_Utility_CONTEXT = 'MAG UTILITY';
  /// RPC to get information from Imaging Site Parameters file
  RPC_MAGGUSER2 = 'MAGGUSER2';
  /// RPC to get network locations
  RPC_MAG_GET_NETLOC = 'MAG UTIL GETNETLOC';
  /// RPC to get list of installed Imaging versions
  RPC_MAG_INSTALL = 'MAGG INSTALL';
  /// RPC get get version status
  RPC_MAG4_VERSION_STATUS = 'MAG4 VERSION STATUS';
  /// RPC get information on 1 Image
  RPC_MAGG_IMAGE_INFO = 'MAGG IMAGE INFO';
  RPC_MAGGUSERKEYS = 'MAGGUSERKEYS';
  RPC_GET_VAR_VALUE = 'XWB GET VARIABLE VALUE';
  RPC_MAG_UTIL_DT2IEN = 'MAG UTIL DT2IEN';
  RPC_MAG_UTIL_CLNLOC = 'MAG UTIL CLNLOC';
  RPC_MAG_UTIL_LSTOFLJB = 'MAG UTIL LSTOFLJB';
  RPC_MAG_UTIL_BKONLJB = 'MAG UTIL BKONLJB';
  RPC_MAG_UTIL_JBOFFLN = 'MAG UTIL JBOFFLN';
  RPC_MAG_STORAGE_FETCH_SET = 'MAG STORAGE FETCH SET';
  RPC_MAG_STORAGE_FETCH = 'MAG STORAGE FETCH';
  RPC_MAGQ_JBQUE = 'MAGQ JBQUE';
  RPC_MAGQBP_UPDATE = 'MAGQBP UPDATE';

Implementation

{*
  Creates a new TMagUtilityBroker
  @param Logger The default logger
  @param ErrorLogger Logger to write errors to
 }

Constructor TMagUtilityBroker.Create(Logger: TLogger; ErrorLogger: TLogger);
Begin
  Inherited Create();
  FBroker := TRPCBroker.Create(Nil);
  FLogger := Logger;
  FErrorLogger := ErrorLogger;
End;

{*
 Destructor destroys all allocated variables
}

Destructor TMagUtilityBroker.Destroy;
Begin
  Inherited;
  If FBroker <> Nil Then
  Begin
    FBroker.Connected := False;
    FreeAndNil(FBroker);
  End;
End;

{*
 Logs into the VistA system
 @param server The server to connect to (can be empty)
 @param port the port to connect to (can be empty)
 @return Result to indicate the status, ok if login is ok, broker error if there
 is a broker error or context failed if the user doesn't have the required context
 }

Function TMagUtilityBroker.LogIn(Var Server, Port: String; Var Errmsg: String): TMagUtilityBrokerLoginResult;
Begin
  Result := MagUtilityLoginError;
  //t17 - flogger.Info(#9 + 'Startup');
  Try
    If (GetServerInfo(Server, Port) = MrOK) Then
    Begin
      FBroker.Server := Server;
      FBroker.ListenerPort := Strtoint(Port);
      FBroker.Connected := True;
      If FBroker.Connected Then
      Begin
        Flogger.Info(#9 + 'Startup' + #9 + #9 + 'Connecting to site (' + Server + ', port: ' + Port + ')' +
                    #9 + 'User: ' + GetUserFullName()); //t17 LOG
        FErrorlogger.Info(#9 + 'Startup' + #9 + #9 + 'Connecting to site (' + Server + ', port: ' + Port +')'+
                    #9 + 'User: ' + GetUserFullName()); //t17 LOG
        //Flogger.Info(#9+'Startup'+#9+#9+'Connected to VistA');
        //Flogger.Info(#9+'Startup'+#9+#9+'Creating context [' + MAG_Utility_CONTEXT + ']');
        If Not FBroker.CreateContext(MAG_Utility_CONTEXT) Then
        Begin
          Errmsg := ',Unable to create context [' + MAG_Utility_CONTEXT +
            '], disconnecting broker';
          FLogger.Error(Errmsg);
          FBroker.Connected := False;
          Result := MagUtilityLoginContextFailed;
        End
        Else
        Begin
          Result := MagUtilityLoginOK;
        End;
      End;
    End;
  Except
    On e: Exception Do
    Begin
      Result := MagUtilityLoginError;
      Flogger.error(#9 + 'Startup' + #9 + #9 + 'TMagUtilityBroker.login, ' + e.Message);
      FErrorLogger.Error(',TMagUtilityBroker.login, ' + e.Message);
      Errmsg := e.Message;
    End;
  End;
End;

{*
 Disconnects the broker connection
 @return true if the disconnect was ok, false if there was an error
 }

Function TMagUtilityBroker.Logout(): Boolean;
Begin
  Try
    If FBroker <> Nil Then
    Begin
      FBroker.Connected := False;
    End;
    Result := Not FBroker.Connected;
  Except
    On e: Exception Do
    Begin
      Result := False;
      flogger.Error(#9 + 'Startup' + #9 + #9 + 'TMagUtilityBroker.Logout, ' + e.Message);
      FErrorLogger.Error(',TMagUtilityBroker.Logout, ' + e.Message);
    End;
  End;
End;

{*
 Returns the connected status
 @return true if connected, false if not connected
 }

Function TMagUtilityBroker.getConnected(): Boolean;
Begin
  If FBroker = Nil Then
    Result := False
  Else
    Result := FBroker.Connected;
End;

{*
  Retrieve the default credentials from the Imaging Site Parameters file
  @param username the username from VistA
  @param password the Password from VistA
  @return True if the RPC ran successfully, false if there was an error
 }

Function TMagUtilityBroker.getSiteParameterCredentials(Var Username,
  Password, PrimarySiteStationNumber: String): Boolean;
Var
  Rlist: TStrings;
Begin
  Try
    Rlist := Tstringlist.Create();
    FBroker.REMOTEPROCEDURE := RPC_MAGGUSER2;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[0].Value := 'MagUtility';
    FBroker.LstCALL(Rlist);
    If Rlist.Count <= 2 Then
    Begin
      FLogger.Error(#9 + 'RPC - Broker' + #9 + #9 + 'Incorrect data recieved from MAGGUSER2 rpc');
      If Rlist.Count > 0 Then
        FLogger.Error(#9 + 'RPC - Broker' + #9 + #9 + 'TMagUtilityBroker.getSiteParameterCredentials, MAGGUSER2 response [' + Rlist[0] + ']');
      Result := False;
      Exit;
    End;
    Username := MagPiece(Rlist[2], '^', 1);
    Password := Copy(Rlist[2], Length(Username), 1000);
    PrimarySiteStationNumber := Rlist[12];
    Result := True;
  Except
    On e: Exception Do
    Begin
      Result := False;
      FLogger.error(#9 + 'RPC - Broker' + #9 + #9 + 'TMagUtilityBroker.getSiteParameterCredentials, ' + e.Message);
      FErrorLogger.Error(#9 + 'RPC - Broker' + #9 + #9 + 'TMagUtilityBroker.getSiteParameterCredentials, ' + e.Message);
    End;
  End;
End;

Function TMagUtilityBroker.getUserInfo(Var Sitecode, uname, Place: String): Boolean;
Var
  Rlist: TStrings;
Begin
  Try
    Rlist := Tstringlist.Create();
    FBroker.REMOTEPROCEDURE := RPC_MAGGUSER2;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[0].Value := 'MagUtility';
    FBroker.LstCALL(Rlist);
    If Rlist.Count <= 2 Then
    Begin
      FLogger.Error(#9 + 'RPC - Broker' + #9 + #9 + 'Incorrect data recieved from MAGGUSER2 - RPC');
      If Rlist.Count > 0 Then
        FLogger.Error(#9 + 'TMagUtilityBroker.getUserInfo' + #9 + 'MAGGUSER2 response [' + Rlist[0] + ']');
      Result := False;
      Exit;
    End;
    Sitecode := MagPiece(Rlist[4], '^', 2);
    uname := MagPiece(Rlist[1], '^', 2); //user full name
    Place := MagPiece(Rlist[4], '^', 1);
    Result := True;
  Except
    On e: Exception Do
    Begin
      Result := False;
      FLogger.error(#9 + 'TMagUtilityBroker.getUserInfo ' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.getUserInfo' + #9 + e.Message);
    End;
  End;
End;

{*
  Retrieves the network locations from VistA
  @param Shares The shares from the VistA database
  @return True if the RPC ran successfully, false if there was an error
 }

Function TMagUtilityBroker.GetNetworkLocations(Var Shares: TStrings): Boolean;
Begin
  If Shares = Nil Then
    Shares := Tstringlist.Create();
  Try
    Result := False; //initialize result value
    FBroker.PARAM[0].Value := 'ALL';
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAG_GET_NETLOC;
    FBroker.LstCALL(Shares);
    If (MagPiece(Shares[0], '^', 1) = '0') Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.getNetworkLocations' + #9 + MagPiece(Shares[0], '^', 2));
      Exit;
    End;
    If (Shares.Count = 0) Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.getNetworkLocations' + #9 + 'Error accessing Network Locations');
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.getNetworkLocations' + #9 + 'Error accessing Network Locations');
      Exit;
    End;
    Shares.Delete(0);
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.getNetworkLocations' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.getNetworkLocations' + #9 + e.Message);
      Result := False;
    End;
  End;
End;

Function TMagUtilityBroker.getRAIDLocations(Var Shares: TStrings): Boolean;
Begin
  Result := False; //initialize result value
  If Shares = Nil Then
    Shares := Tstringlist.Create();
  Try
    FBroker.ClearParameters := True;
    FBroker.PARAM[0].Value := 'MAG';
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAG_GET_NETLOC;
    FBroker.LstCALL(Shares);
{* mag get loc - data piece
  Piece # 1: ien
  Piece # 2: \\server\share (without the trailing \)
  Piece # 3: storage type
  Piece # 4: operational status
  Piece # 5: username
  Piece # 6: password
  Piece # 7: MUSE site#
  Piece # 8: Network location SITE
  Piece # 9: place
}
    If (MagPiece(Shares[0], '^', 1) = '0') Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.getRAIDLocations' + #9 + MagPiece(Shares[0], '^', 2));
      Exit;
    End;
    If (Shares.Count = 0) Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.getRAIDLocations' + #9 + 'Error accessing Network Locations');
      FErrorLogger.Error(',TMagUtilityBroker.getRAIDLocations, Error accessing Network Locations');
      Exit;
    End;
    Shares.Delete(0);
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.getRAIDLocations' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.getRAIDLocations' + #9 + e.Message);
      Result := False;
    End;
  End;
End;

{*
  Get the variable value from VistA Broker call
  }

Function TMagUtilityBroker.FnmgetIEN(Var chkvar: String): Integer;
Var
  varvalue: TStrings;
  Value: String;
Begin
  Result := 0; //initialize result value
  Value := '';
  If chkvar = '' Then Exit;
 varvalue := Tstringlist.Create();
 try //T24
  Try
    FBroker.ClearParameters := True;
    FBroker.PARAM[0].Value := chkvar;
    FBroker.PARAM[0].PTYPE := Reference;
    FBroker.REMOTEPROCEDURE := RPC_GET_VAR_VALUE;
    FBroker.LstCALL(varvalue);
    Value := varvalue[0];
    If Value <> '' Then Result := Strtoint(Value);
    //showmessage(value);
    varvalue.Delete(0);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.FnmgetIEN ' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.FnmgetIEN ' + #9 + e.Message);
    End;
  End;
 Finally
   FreeAndNil(varvalue);
 end; //t24 - try (memory release)
End;

{*
  Get the global variable value from VistA Broker call
  }

Function TMagUtilityBroker.FnmGetNode(Var chkvar: String): String;
Var
  varvalue: TStrings;
Begin
 Result := ''; //initialize result value
 If chkvar = '' Then Exit;
 varvalue := Tstringlist.Create();
 try
  Try
    FBroker.ClearParameters := True;
    FBroker.PARAM[0].Value := chkvar;
    FBroker.PARAM[0].PTYPE := Reference;
    FBroker.REMOTEPROCEDURE := RPC_GET_VAR_VALUE;
    FBroker.LstCALL(varvalue);
    Result := varvalue[0];
    varvalue.Delete(0);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.FnmgetNode ' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.FnmgetNode' + #9 + e.Message);
    End;
  End;
 Finally
   FreeAndNil(varvalue);
 end; //T24 - try (memory release
End;

{*
  RPC_MAG_UTIL_DT2IEN - USE DATE TO FIND first IEN range
  }

Function TMagUtilityBroker.DTRangeIEN(Var DTrange, limit: String): String;
Var
  varvalue: TStrings;
Begin
  Result := ''; //initialize result value
  If DTrange = '' Then Exit;
  varvalue := Tstringlist.Create();
  Try
    FBroker.ClearParameters := True;
    FBroker.PARAM[0].Value := DTrange;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[1].Value := limit;
    FBroker.PARAM[1].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAG_UTIL_DT2IEN;
    FBroker.LstCALL(varvalue);
    Result := varvalue[0];
    If Strtoint(MagPiece(Result, ',', 1)) > 0 Then DTrange := varvalue[1];
    If (limit='1') And (StrToDatetime(varvalue[1])>StrToDatetime(varvalue[2])) Then  //T21 CR 0305
    Begin
        //showmessage('Cannot find any image files after '+ DTrange +', but on '+ varvalue[2]);  //t21 CR 0305
        showmessage('There are no image files on or after the entered date ('+ DTrange +').  The last image file is dated '+ varvalue[2]+'.'); //T24
        Result := '0';  DTrange := '';
    End;
    varvalue.Delete(0);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.DTRangeIEN' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.DTRangeIEN' + #9 + e.Message);
    End;
  End;
End;

{*
  Remote Procedure VistA - Search failed GCC BP queue and try re-queue it
 }


Function TMagUtilityBroker.NtLocDel(Var Ien, Place, Stp: String): String;
Var
  varvalue: TStrings;
  no: Integer;
Begin
 If (Ien = '') Or (Place = '') Then Exit;
 varvalue := Tstringlist.Create();
 Result := '';
 //showmessage(stp);
 try
  Try
    FBroker.ClearParameters := True;
    FBroker.ClearResults := True;
    FBroker.PARAM[0].Value := Ien;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[1].Value := Place;
    FBroker.PARAM[1].PTYPE := LITERAL;
    FBroker.PARAM[2].Value := Stp;
    FBroker.PARAM[2].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAG_UTIL_CLNLOC;
    FBroker.LstCALL(varvalue);
    Result := varvalue[0];
    //showmessage(result);
    If (varvalue.Count > 1) And (MagPiece(Result, ',', 1) = '-1') Then
    Begin
      For no := 1 To varvalue.Count - 1 Do
      Begin
        Result := Result + '; ' + MagPiece(varvalue[no], ',', 2);
        FLogger.Error(#9 + 'Network Loc Cleanup' + #9 + MagPiece(varvalue[no], ',', 2));
        FErrorLogger.Error(#9 + 'Network Loc Cleanup' + #9 + MagPiece(varvalue[no], ',', 2));
        Exit;//T18
      End;
    End;
    varvalue.Delete(0);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'MagUtilityBroker.NTlOcDel, (' + Ien + ', ' + Place + ')' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'MagUtilityBroker.NtLocDel, (' + Ien + ', ' + Place + ')' + #9 + e.Message);
    End;
  End;
 Finally
  FreeAndNil(varvalue);
 end;  //T24 - try to stop memory release
End;

Function TmagUtilityBroker.MagFindfile(Magien, Filetype: String): String;
Var
  varvalue: TStrings;
Begin
  Result := '';
  varvalue := Tstringlist.Create();
 try
  Try
    FBroker.REMOTEPROCEDURE := RPC_MAGG_IMAGE_INFO;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[0].Value := Magien;
    FBroker.PARAM[1].PTYPE := LITERAL;
    FBroker.PARAM[1].Value := '1';
    FBroker.LstCALL(varvalue);
    Result := varvalue[0];
    varvalue.Delete(0);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.FindFile ' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.FindFile ' + #9 + e.Message);
    End;
  End;
 Finally
   FreeAndNil(varvalue);
 end; //T24 - try to stop memory release
End;

Function TmagUtilityBroker.MagInstall(Var Version: String): Boolean;
Var
  varvalue: TStrings;
  no: Integer;
Begin
  Result := False;
  varvalue := Tstringlist.Create();
  Try
    FBroker.REMOTEPROCEDURE := RPC_MAG_INSTALL;
    FBroker.LstCALL(varvalue);
    For no := 0 To varvalue.Count - 1 Do
      If Version = MagPiece(varvalue[no], '^', 1) Then Result := True;
    varvalue.Delete(0);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.Magginstall ' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.Magginstall ' + #9 + e.Message);
    End;
  End;
End;

Function TmagUtilityBroker.setNETloc(Magien, LocIEN: String): Boolean;
Var
  MAGR: TStrings;
Begin
  Result := False; //initialize result value
  MAGR := Tstringlist.Create();
 try
  Try
    FBroker.PARAM[0].Value := Magien;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[1].Value := LocIEN;
    FBroker.PARAM[1].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAG_STORAGE_FETCH_SET;
    FBroker.LstCALL(MAGR);
    If (MagPiece(MAGR[0], ',', 1) <> '1') Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.setLocationIEN, ' + MagPiece(MAGR[0], '^', 2) + ', ' + Magien + ' ' + LocIEN);
      Exit;
    End;
    If (MAGR.Count = 0) Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.setLocationIEN, Error accessing Network Locations');
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.setLocationIEN, Error accessing Network Locations');
      Exit;
    End;
    MAGR.Delete(0);
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.setLocationIEN ' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.setLocationIEN ' + #9 + e.Message);
      Result := False;
    End;
  End;
 Finally 
   FreeAndNil(MAGR);
 end;//T24
End;

///

Function TmagUtilityBroker.unsetNETloc(Filename, filextn: String; Magien: Integer): Boolean;
Var
  MAGR: TStrings;
Begin
  Result := False; //initialize result value
  MAGR := Tstringlist.Create();
 try
  Try
    FBroker.PARAM[0].Value := Filename;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[1].Value := Filextn;
    FBroker.PARAM[1].PTYPE := LITERAL;
    FBroker.PARAM[2].Value := Inttostr(Magien);
    FBroker.PARAM[2].PTYPE := LITERAL;
    FBroker.PARAM[3].Value := 'NET';
    FBroker.PARAM[3].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAGQBP_UPDATE;
    FBroker.LstCALL(MAGR);
    If (MagPiece(MAGR[0], ',', 1) <> '1') Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.unsetLocationIEN ' + #9 + MagPiece(MAGR[0], '^', 2));
      Exit;
    End;
    If (MAGR.Count = 0) Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.unsetLocationIEN, Error accessing Network Locations');
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.unsetLocationIEN, Error accessing Network Locations');
      Exit;
    End;
    MAGR.Delete(0);
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.unsetLocationIEN, ' + e.Message);
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.unsetLocationIEN, ' + e.Message);
      Result := False;
    End;
  End;
 Finally
   FreeAndNil(MAGR);
 end; //T24 - try (memory release)
End;

Function TmagUtilityBroker.queueJB(Magien: String): Boolean;
Var
  MAGR: TStrings;
Begin
  Result := False; //initialize result value
  MAGR := Tstringlist.Create();
  Try
    FBroker.ClearParameters := True;
    FBroker.ClearResults := True;
    FBroker.PARAM[0].Value := Magien;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAGQ_JBQUE;
    FBroker.LstCALL(MAGR);
    If (MagPiece(MAGR[0], ',', 1) <> '0') Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.queueJB,' + MagPiece(MAGR[0], '^', 2));
      Exit;
    End;
    If (MAGR.Count = 0) Then
    Begin
      FLogger.Error(#9 + 'TMagUtilityBroker.queueJB, Error accessing Network Locations');
      FErrorLogger.Error(#9 + 'TMagUtilityBroker.queueJB, Error accessing Network Locations');
      Exit;
    End;
    MAGR.Delete(0);
    FLogger.Info(#9 + 'Orphan' + #9 + #9 + 'Orphan file is found No-Archive image, and queued Jukebox ' + MAGR[0]);
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'Orphan' + #9 + #9 + 'TMagUtilityBroker.queueJB, ' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'Orphan' + #9 + #9 + 'TMagUtilityBroker.queueJB, ' + e.Message);
      Result := False;
    End;
  End;
End;

{*
  Returns the connected users full name. If the broker is nil or not connected
  then this returns an empty string
  @return The connected users name
 }

Function TMagUtilityBroker.GetUserFullName(): String;
Begin
  Result := '';
  If FBroker = Nil Then
    Exit;
  If Not FBroker.Connected Then
    Exit;
  Result := FBroker.User.Name;
End;

{*
  RPC to mark image 2005 file offline base on off-shelf JB platter TXT report
  }

Function TMagUtilityBroker.JBOffLn(Var TxtLn, Platter: String): Boolean;
Var
  MagR: TStrings;
  Msg: String;
Begin
  Result := False;
  Msg := '';
  MagR := Tstringlist.Create(); //Return result from RPC
 try
  Try
    FBroker.ClearParameters := True;
    FBroker.ClearResults := True;
    FBroker.PARAM[0].Value := TxtLn;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[1].Value := Platter;
    FBroker.PARAM[1].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAG_UTIL_JBOFFLN;
    FBroker.LstCALL(MagR);
    If MagR[0] = '0' Then Exit;
    If MagPiece(MagR[0], ',', 1) = '0' Then Result := True;
    If MagPiece(MagR[0], ',', 1) = '-1' Then Msg := MagPiece(MagR[0], ',', 2);
    MagR.Delete(0);
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'Offline Juke' + #9 + 'TMagUtilityBroker.JBOffLn, ' + e.Message + Msg);
      FErrorLogger.Error(#9 + 'Offline Juke' + #9 + 'TMagUtilityBroker.JBOffLn, ' + e.Message + Msg);
      Result := False;
    End;
  End;
 Finally 
   FreeAndNil(MagR);
 end;//T24 - try (stop memory release)
End;

{*
  Remote Procedure Call to list the current Offline JB image platter(s)
  }

Function TMagUtilityBroker.LstOflJB(Var JBlst: TStrings): Boolean;
Var
  MAGR: TStrings;
Begin
  MAGR := Tstringlist.Create();
  Try
    FBroker.REMOTEPROCEDURE := RPC_MAG_UTIL_LSTOFLJB;
    FBroker.LstCALL(MAGR);
    JBlst := MAGR;
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'Online Juke' + #9 + 'TMagUtilityBroker.LstOflJB' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'Online Juke' + #9 + 'TMagUtilityBroker.LstOflJB' + #9 + e.Message);
      Result := False;
    End;
  End;
End;

{*
  remote Procedure to take the Platter's images (offline) back online
  }

Function TMagUtilityBroker.BKOnlJB(Var OflJB: String): Boolean;
Var
  MAGR: TStrings;
  cnt : String;
Begin
  Result := False; //initialize result value
  MAGR := Tstringlist.Create();
 try
  Try
    FBroker.REMOTEPROCEDURE := RPC_MAG_UTIL_BKONLJB;
    FBroker.PARAM[0].Value := OflJB;
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.LstCALL(MAGR);
    ///showmessage(MAGR[0]);
    If MagPiece(MAGR[0], ',', 1) = '0' Then
    Begin
      Result := True;
      cnt := MagPiece(MAGR[0], ',', 2); //Done_cnt
      cnt := MagPiece(cnt, ' ', 2); //cnt
      //T18 - FLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked total '+cnt+' files online'); //t17 - LOG, info log #2
      //T18 - FErrorLogger.Info(#9 + 'Offline Jukebox Images' + #9 + 'Marked total '+cnt+' files online'); //t17  - LOG , info log #2
    End;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'Online Jukebox Images' + #9 + 'TMagUtilityBroker.BackOnlineJB' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'Online Jukebox Images' + 'TMagUtilityBroker.BackOnlineJB' + #9 + e.Message);
      Result := False;
    End;
  End;
 Finally
   FreeAndNil(MAGR);
 end;//T24 - try (memory release)
End;

{*
 Gets the keys the user has on the VistA system
 @param keys The list of keys the user has assigned
 @return True if the RPC ran successfully, false if there was an error
 }

Function TMagUtilityBroker.getUserKeys(Var Keys: Tstringlist): Boolean;
Begin
  Keys := Tstringlist.Create();
  Try
    FBroker.REMOTEPROCEDURE := RPC_MAGGUSERKEYS;
    FBroker.LstCALL(Keys);
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'Startup' + #9 + #9 + 'TMagUtilityBroker.getUserKeys' + #9 + e.Message);
      FErrorLogger.Error(#9 + 'Startup' + #9 + #9 + 'TMagUtilityBroker.getUserKeys' + #9 + e.Message);
      Result := False;
    End;
  End;
End;

{*
  FUT49 - Image File Storage Copy
 }

Function TmagUtilityBroker.FETCH(Var MAGR: TStrings; Magien, LocIEN: String): Boolean;
Begin
  MAGR := Tstringlist.Create();
  Try
    FBroker.ClearParameters := True;
    FBroker.ClearResults := True;
    FBroker.PARAM[0].Value := Magien; //ien range (ien1:ien2)
    FBroker.PARAM[0].PTYPE := LITERAL;
    FBroker.PARAM[1].Value := ' '; //date range (not use in P98)
    FBroker.PARAM[1].PTYPE := LITERAL;
    FBroker.PARAM[2].Value := LocIEN; //network location ien
    FBroker.PARAM[2].PTYPE := LITERAL;
    FBroker.REMOTEPROCEDURE := RPC_MAG_STORAGE_FETCH;
    FBroker.LstCALL(MAGR);
    Result := True;
  Except
    On e: Exception Do
    Begin
      FLogger.Error(#9 + 'Storage Copy' + #9 + 'TMagUtilityBroker.FETCH_FUT49, Exception error' + e.Message);
      FErrorLogger.Error(#9 + 'Storage Copy' + #9 + 'TMagUtilityBroker.FETCH_FUT49, Exception error ' + e.Message);
      Result := False;
    End;
  End;
End;

{*
  Return the VistA context required to run MagUtility
  @return The context
 }

Function TMagUtilityBroker.getRequiredContext(): String;
Begin
  Result := MAG_Utility_CONTEXT;
End;

{*
  Retrieves the list of installed versions of VistA Imaging
  @param Fstat the status of the RPC call
  @param Flist the list of installed versions of Imaging
 }

Procedure TMagUtilityBroker.RPMaggInstall(Var Fstat: Boolean; Var Flist: Tstringlist);
Begin
  If (FBroker = Nil) Or (Not FBroker.Connected) Then
  Begin
    Flist.Add('0^No connection to the Server');
    Exit;
  End;
  Flist.Clear;
  Fstat := False;
  FBroker.REMOTEPROCEDURE := RPC_MAG_INSTALL;
  Try
    FBroker.LstCALL(Flist);
    If (Flist.Count = 0) Or (MagPiece(Flist[0], '^', 1) = '0') Then Exit;
    Fstat := True;
    If Maglength(Flist[0], '~') = 1 Then // This is old M code, not returning '~' codes for columns;
      Flist[0] := 'Server Versions  ^Install Date     ~S1'; // this sort code 'S1' says that this column is Dates.
  Except
    On e: Exception Do
    Begin
      Flist.Insert(0, '0^' + MagPiece(e.Message, #$A, 2));
      FErrorLogger.Error(#9 + 'Startup' + #9 + 'TMagKatBroker.restartMagScanImageTextFiles, ' + e.Message);
      FLogger.Error(#9 + 'Startup' + #9 + 'TMagKatBroker.restartMagScanImageTextFiles, ' + e.Message);
    End;
  End;
End;

{*
  Retrieves the version status from VistA
  @param status the status response message
  @param version The version
 }

Procedure TMagUtilityBroker.RPMagVersionStatus(Var Status: String; Version: String);
Begin
  If (FBroker = Nil) Or (Not FBroker.Connected) Then
  Begin
    Status := '1^Status is unavailable.';
    Exit;
  End;
  FBroker.REMOTEPROCEDURE := RPC_MAG4_VERSION_STATUS;
  FBroker.PARAM[0].PTYPE := LITERAL;
  FBroker.PARAM[0].Value := Version;

  Try
    Status := FBroker.STRCALL;
    If (Status = '') Then
    Begin
      Status := '1^Error trying to determine status';
      FLogger.Error(#9 + 'Startup' + #9 + #9 + 'Error Checking Status of Version: ' + Version + '. ,');
      FErrorLogger.Error(#9 + 'Startup' + #9 + 'Error Checking Status of Version: ' + Version + '. ,');
      Exit;
    End;
  Except
    On e: Exception Do
    Begin
      Status := '1^Error trying to determine status';
      FErrorLogger.Error(',TMagUtilityBroker.RPMagVersionStatus, ' + e.Message + '. ,');
      FLogger.Error(#9 + 'Startup' + #9 + 'TMagUtilityBroker.RPMagVersionStatus, ' + e.Message + '. ,');
    End;
  End;
End;

End. //cMagUtilityBroker
